#ifndef CINESPACE_H
#define CINESPACE_H
#include "cinesim.h"
#include "espace/satgeo.h"
#include <vector>
//using namespace std; // pour vector

// utilitaire copy fichier ressource pour acces depuis les vieilles routines en C
bool cpyRes(const QString &res,const char *nomfich);

// Différents scenarios d'avancement du temps sont simulés : Une boite de dialogue permet de définir :
//  - une date début et de fin de la simulation
//  - découpée en périodes de durées égales définie en jours, et dans chaque période :
//  - seul le premier créneau de durée défini en secondes est simulée, le reste de la période est sauté.
// NOSCENE : Simulation suivant exactement les paramètres ci-dessus
// LEVER : Les débuts de tranches sont pris au lever du Soleil le plus proche
// COUCHER : Les débuts de tranches sont pris au coucher du sSoleil le plus proche
typedef enum {NOSCENE, LEVER, COUCHER} Scenario;

typedef enum {RIEN, STARS, EARTH, LES2} ModeRec;

class TimeDialog;
class ConfDialog;
class ConfStarsDialog;
class ConfEarthDialog;
class TerreGLWidget;
class UnivGLWidget;
class StarsWidget;
class Etoile;
class Constellation;

class CineSpace : public CineSim
{
    Q_OBJECT
public:
    explicit CineSpace(QWidget *parent = 0);
    friend class TimeDialog;
    friend class ConfDialog;
    friend class ConfStarsDialog;
    friend class ConfEarthDialog;
    friend class TerreGLWidget;
    friend class UnivGLWidget;
    friend class StarsWidget;

    double temps() { return time; }

protected:
    TimeDialog *timeDialog;
    ConfDialog *confDialog;
    Scenario scenar;
    double precision;
    double time; // secondes par / à J2000.
    double tdeb, tfin; // secondes / à J2000.
    double periode ; // Découpage artificiel du temps simu en périodes
    double creneau; // Intervalle de temps affiché par période
    double decalage; // Décalage temporel ajouté au temps simu, uniquement utile pour les scenarios lever et coucher
    double accel; // Accélération du temps (durée de temps simulée en 1 seconde usager)
    double dtsim; // intervalle de temps simu entre 2 affichages
    double deltaCreneau; // Decalage permettant de recaler le début de la tranche avec un lever ou coucher
    std::vector<TwoLE *> *all_tle;
    std::vector<SatGeo *> *all_sats;
    int satcou;
    int memTranche;
    void majCine() {update(time);}

private :

    // Ajout fonction enregistrement au CommandWidget
    bool recdown;        // Pour le clignotement du bouton record
    ModeRec modeRec;    // RIEN, STARS EARTH LES2
    QTimer *clignoTimer; // timer clignotement bouton enregistrement
    char *repEarth;      // Répertoire enregistrement video
    char *fileEarth;      // Fichier enregistrement video
    char *strNumImgEarth; // Num image dans fichier enregistrement video
    int numImgEarth;
    char *repStars;      // Répertoire enregistrement video
    char *fileStars;      // Fichier enregistrement video
    char *strNumImgStars; // Num image dans fichier enregistrement video
    int numImgStars;

    QLabel *labelHelp;
    Etoile *tabStars;
    Constellation *tabConstellations;
    UnivGLWidget *winUniv;
    TerreGLWidget *winTerre;
    StarsWidget *winStars;
    void update(double t);

private slots:
    void afficheHelpSlot() {labelHelp->show();}
    void showTimeDialogSlot();
    void showConfDialogSlot();
    void synchroTimeDialogSlot();
    void showStarsSlot();
    void showTerreSlot();
    void showUnivSlot();
    void majSceneSlot(int);
    void saveSceneSlot();
    void loadSceneSlot();   
    // Pour l'enregistrement
    void clignoRecordSlot();
    void recordSlot();


protected:
    void closeEvent(QCloseEvent *event);
};

#endif // CINESPACE_H
