#include <QtGui>
#include <QAction>
#include <QFileDialog>
#include "TerreGLWidget.h"
#include "../cinespace.h"
#include "confearthdialog.h"
#include <math.h>
#include "terre.h"
#include "reperes.h"
#include "lune.h"
#include "satgeo.h"
#include "../grafutils/qtglutils.h"
#include "../mathutil/rotation.h"
#include "../mathutil/sp3.h"
extern "C" {
GLuint glNewPleiade(GLfloat size);
}

#if 0
static double gOmega[8] = {0.843, 1.3387, 0., 0.8657, 1.7523, 1.984, 1.2915,   2.3};
static double inclin[8] = {0.12217, 0.05934, 0., 0.03229, 0.02278, 0.04335, 0.01344, 0.03089};
static double pomega[8] = {0.50824, 0.95731, 1.79594, 5.00037, 4.78220, 5.91318, 1.71042, 4.81711};
static double demgaxe[8] = {0.387, 0.723, 1., 1.524, 5.202, 9.555, 19.2, 30.1};
static double excen[8] = {0.206, 0.0068, 0.0167, 0.093, 0.048, 0.055, 0.046, 0.0086};
static GLdouble colOrb[8][3] = {{0.2, 0.2f, 0.2f},
                               {1.0, 1.0, 0.0},
                               {0.47, 0.71, 1.0},
                               {1.0, 0.0, 0.0},
                               {7.0, 7.0, 0.2},
                               {1.0, 1.0, 0.0},
                               {0.0, 1.0, 0.0},
                               {0.0, 0.0, 1.0}};
#endif

// Quelques intensités pour les éclairages openGL
static GLfloat eteinte[] = {0.0f, 0.0f, 0.0f, 1.0f};
static GLfloat tenue[] = {0.1f, 0.1f, 0.1f, 1.0f};
static GLfloat faible[] = {0.2f, 0.2f, 0.2f, 1.0f};
static GLfloat moyenne[] = {0.5f, 0.5f, 0.5f, 1.0f};
static GLfloat forte[] = {0.8f, 0.8f, 0.8f, 1.0f};
static GLfloat puissante[] = {1.0f, 1.0f, 1.0f, 1.0f};

// La classe d'affichage du Widget openGL de la scene spatiale
// -----------------------------------------------------------
TerreGLWidget::TerreGLWidget(CineSpace *c)
{
    this->setWindowTitle("Vue 3D voisinage Terre");
    cine = c;

    // Génération des éléments du menu contextuel
    setContextMenuPolicy(Qt::ActionsContextMenu);

    //Item pour l'appel de la boite de dialogue
    QAction *confAction = new QAction(tr("&Configuration"), this);
    connect(confAction, SIGNAL(triggered()), this, SLOT(confDialogSlot()));
    addAction(confAction);

    //Item pour le "Help"
    labelHelp = new QLabel(
                tr(
                    "<h4>Action touches clavier :</h4>"
                    "<ul><li>Flèches haut/bas : Rotation en site.</li>"
                    "<li>Flèches droite/gauche : Rotation en azimut.</li>"
                    "<li>Lettre E/e : Eloignement ou rapprochement.</li>"
                    "<<li>Echappement : Retour point de vue initial.</li>"
                    "</ul>"
                    "<h4>Menu contextuel :</h4>"
                    "<ul><li>Configuration :"
                    "<br/>Affiche une boite de dialogue permettant de sélectionner divers points de vue.</li>"
                    "<li>Sauve Image :"
                    "<br/>Permet de sauvegarder la vue de la fenêtre en image jpeg.</li>"
                    "<li>Help :"
                    "<br/>Affiche cette aide.</li>"
                    "</ul>"
                    ));
    labelHelp->setWindowTitle("Aide Vue Terre");

    // Le Slot sauver image
    QAction *sauveImg = new QAction(tr("Sauve Image"), this);
    connect(sauveImg, SIGNAL(triggered()), this, SLOT(grabImageSlot()));
    addAction(sauveImg);

    QAction *actionHelp = new QAction(tr("&Help"), this);
    connect(actionHelp, SIGNAL(triggered()), this, SLOT(afficheHelpSlot()));
    addAction(actionHelp);

    // Format de la fenêtre OpenGL
    setFormat(QGLFormat(QGL::DoubleBuffer | QGL::DepthBuffer | QGL::Rgba));

    // Initialisation paramètre de la scene dans le repère openGL initial
    perspective = 1;            // Flag utilisation perspective
    //    AngleVueVertical = 30. ;    // Champ de vue en degrés
    modePtVue = toEARTH_c;        // Point de vue, modifiable par la boite de dialogue
    gainDist = 1.0;             // Pour éloigner ou approcher le point de vue (déplacement camera)
    gainAngle = 1.0;             // Pour éloigner ou approcher le point de vue (zoom)
    sitScene = sitSceneIni = 0.0; // Pour faire bouger la scene en site
    aziScene = aziSceneIni = 0.0; // Pour faire bouger la scene en azimut
    drawLune = true;
    drawSatellites = false;

    //  On fabrique la boite de dialogue de configuration de la scene à afficher
    confEarthDialog = new ConfEarthDialog(this, this);

}

// Enregistrement d'image-ecran sous format jpg
void TerreGLWidget::grabImageSlot()
{
    QImage image = grabFrameBuffer();
    QString filename = QFileDialog::getSaveFileName
            (this,tr("Sauve Image"),".",tr("Fichier image (*.jpg)"));
    if (!filename.isEmpty()) image.save(filename);
}

// Reception clic contextuel pour l'appel boite de dialigue configuration
void TerreGLWidget::confDialogSlot()
{
    confEarthDialog->setModal(false);
    confEarthDialog->show();
}

// Dimension par defaut de la fenetre OpenGl
QSize TerreGLWidget::sizeHint() const
{
    return QSize(720,720);
}

// Si fermeture on fait le ménage
void TerreGLWidget::closeEvent(QCloseEvent *event)
{
    if(confEarthDialog) confEarthDialog->close();
    labelHelp->close();
}


// Initialisation OpenGL
void TerreGLWidget::initializeGL()
{
    //  Couleur d'effacement sombre
    glClearColor(0.05, 0.05, 0.05, 1.0);
    glEnable(GL_DEPTH_TEST); // Elimination des faces cachees
    glFrontFace(GL_CCW); // Les faces avant sont celles qui sont parcourues dans le sens direct (par défaut a priori)
    glEnable(GL_CULL_FACE); // Les faces avant ou arrières seront pré-éliminées
    glCullFace (GL_BACK) ; // Ce sont les Faces arrières qui seront pré-éliminées (par défaut a priori)

    // Définition des composantes de la lumiere GL_LIGHT0 utilisée comme lumière solaire
    glLightfv(GL_LIGHT0, GL_AMBIENT, eteinte);
    glLightfv(GL_LIGHT0, GL_DIFFUSE, puissante);
    glLightfv(GL_LIGHT0, GL_SPECULAR, moyenne);
    glEnable(GL_LIGHT0);

    // Renvoi la couleur par le matériel en mode color-tracking
    glEnable(GL_COLOR_MATERIAL);

    // Amélioration des calculs de l'éclairage
    glLightModelfv(GL_LIGHT_MODEL_AMBIENT, tenue); // Lulière ambiante globale atténuée pour faire la nuit coté nuit.
    glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR); // Pour améliorer le rendu des textures
    glShadeModel(GL_SMOOTH); // Pour un degradé des couleurs (inutile ? car par défaut)

    // Fabrication de la terre texturée dans une DisplayListe
    dl_Terre = newDlPanete((float) Terre::rT, 48, ":/laTerre");
    // Fabrication de la lune texturée dans une DisplayListe
    dl_Lune = newDlPanete((float) Lune::rayLune, 24, ":/laLune");
    // Fabrication d'un satellite de type pleiade dans une DisplayListe
    dl_Sat = glNewPleiade(1.0f);
#if 0
    // Fabrication des orbites
    for (int i = 0; i <8; i++)
        dl_Orbs[i] = glNewOrbite(demgaxe[i]*Terre::UA, excen[i], inclin[i], gOmega[i], pomega[i], 60);
#endif
}

// Si redimmensionnement de la fenêtre, on recalcule les paramètres de projection
void TerreGLWidget::resizeGL(int w, int h)
{
    width1 = w;  height1 = h;
    glViewport(0,0,width1,height1); // On affiche sur toute la fenêtre
    setProjection();
}

/*----------------------------------------------------------------------------------------------
 * Préparation des matrices de projection et scène. On prend en compte les paramètres suivants :
 *   taille de la fenêtre : width1 et height1
 *   modePtVue : le mode de point de vue désiré
 *   AngleVueIni : si grand, effet grand angle, si petit effet zoom.
 *   gainAngle : Coefficient multiplcateur de cet angle
 *   Perspective : booleen précisant si proj avec perspective, ou sans ( orthogonale)
 *   ditSceneIni : distance entre l'observateur et le repère scène
 *   gainDist : coef multiplicateur sur cette distance
 *   aziScene, sitScene : direction du regard de l'observateur
 *--------------------------------------------------------------------------------------
 * On considère les repère suivants : Le repère Oeil (OpenGl) et divers repères scène. Le
 * repère openGL initial est centré Oeil, X vers la droite, Y vers le haut et
 * Z vers l'avant de l'écran. Dans cette routine on positionne au moyen de "distSceneIni"
 * le repère scène initial, puis on lui donne son orientation qui dépend du cas traité :
 * cas frSATEL : L'oeil est derrière le satelite et regarde vers la Terre. Le repère scène initial
 *    est le repère satellite qui est orienté avec X à droite et Z vers l'écran (X est selon
 *    la vitesse satellite et Z est vers le centre Terre),
 * cas frSOLEIL : L'oeil est du coté du soleil et regarde vers la Terre. Le repère scène initial
 *    est centré Terre avec X vers l'observateur (soleil) et Z vers le haut de l'écran (normal
 *    à l'écliptique),
 * cas frZENITH : L'oeil est à la verticale du Zénith d'un lieu d'observation. Le repère scène
 *    initial est centré Terre avec Z vers l'oeil (Zénith du lieu) et X vers le bas (Sud du lieu d'observation),
 * cas toEARTH_C : L'oeil est dans l'espace, sur la direction du point Gamma par rapport
 *    à la Terre et regarde vers la Terre. Le repère scène initial est le repère céleste centré
 *    Terre, avec X vers l'oeil (cad Gamma), Z vers le haut (nord).
 * cas toEARTH_E : L'oeil est dans l'espace, sdans la direction du point Gamma par rapport
 *    à la Terre et regarde vers la Terre. Le repère scène initial est le repère écliptique centré
 *    Terre, avec X vers l'oeil (cad Gamma), Z vers le haut (normal à l'écliptique).
 * cas toLUNE_C : Loeil est du coté de la Terre et regarde vers la Lune avec l'axe des Z selon
 *    l'axe des pôles
 * cas toLUNE_A : Loeil est du coté de la Terre et regarde vers la Lune avec l'axe des Z selon
 *    le zénith de l'observateur
 *************************************************************************************/
void TerreGLWidget::setProjection()
{
    int nbsat = cine->all_sats->size();
    double zProche = 1., zInfini = 10.*Lune::dist0;

    ModePtVue mode = modePtVue;
    if (mode == frSATEL && (nbsat <= 0 || !drawSatellites)) mode = toEARTH_c;
    if ((mode == toLUNE_C ||mode == toLUNE_A ) && !drawLune) mode = toEARTH_c;

    // On fixe la distance de l'observateur à la scène initiale en fonction du point de vue
    //-------------------------------------------------------------------------------------

    switch (mode)
    {
    case frSATEL:
        AngleVueIni = 90.;
        distSceneIni = 10.; // Point de vue 10m en avant du satellite
        zProche = 1.;
        zInfini = 1000.*Terre::UA;
        break;
    case toLUNE_A:
    case toLUNE_C:    // Scene centre Lune, oeil au niveau de la Terre
        AngleVueIni = 0.8;
        distSceneIni = Lune::dist0;
        zProche = Lune::dist0/100.;
        zInfini = 1000.*Terre::UA;
        break;
//    case SOLEIL :
//        AngleVueIni = 0.005; // Pour que la Terre soit visible
//        distSceneIni = 0.5*Terre::UA; // Pt de vue derrière le Soleil
//        zProche = Terre::raySol;
//        zInfini = 1000.*Terre::UA;
//        break;
    case frSOLEIL:
    case frZENITH:  // idem céleste
    case toEARTH_E : // idem céleste
    case toEARTH_c :
        AngleVueIni = 1.;
        distSceneIni = 3.*Lune::dist0; // Point de vue 3 rayons terrrestre en avant de la Terre
        zProche = Terre::rT/10.;
        zInfini = 1000.*Terre::UA;
        break;
#if 0
    case SYS_SOL :
        AngleVueIni = 45.;
        distSceneIni = 3.*Terre::UA; // Point de vue 3 distance Terre-Soleil
        zProche = Terre::rT;
        zInfini = 1000.*Terre::UA;
        break;
#endif
    }
    // Définition de la matrice de projection initale (projetant la vue da
    glMatrixMode(GL_PROJECTION);
    glLoadIdentity(); // modifiée ci-après en fonction de l'angle de vue et du mode perspective ou orthogonal
    if(perspective)
        gluPerspective(gainAngle*AngleVueIni, 1.*width1/height1, zProche, zInfini);
    else
    {
        double Dy = gainDist*distSceneIni*tan(0.5*gainAngle*AngleVueIni*M_PI/180.);
        double Dx = Dy*width1/height1;
        glOrtho(-Dx, Dx, -Dy, Dy, zProche, zInfini);
    }
    // Définition de la matrice transformant le repère observateur (OpenGL) en repère scène
    glMatrixMode(GL_MODELVIEW);
    glPopMatrix(); // Pour eviter de saturer la pile avec le push à la fin, si plusieurs appels consécutifs
    glLoadIdentity(); // modifiée ci-après en fonction de sa situation par rapport au rep. observateur
    glTranslated(0., 0., -gainDist * distSceneIni); // Eloigne la scene de l'oeil (Z est vers l'avant ds Rep openGL)

    // On oriente le repère scène initial en fonction du cas traité
    //--------------------------------------------------------------
    switch (mode)
    {
    case frSATEL: // Rscène = Rlocal satellite avec X à droite, Y vers le bas et Z vers l'écran (vers la Terre).
        glRotated(aziScene, 0.0, 1.0, 0.0); // Pour l'exploration en azimut
        glRotated(180.+sitScene, 1.0, 0.0, 0.0); // Retourne Z vers l'écran (et le penche d'un angle sitScene)
        break;
    case frZENITH: // Rscène avec Z zénith et X vers le bas
        glRotated(-90, 0.0, 0.0, 1.0); // Ramène X vers le bas.
        glRotated(aziScene, 1.0, 0.0, 0.0); // Rotation en azimut autour vertical écran.
        glRotated(sitScene, 0.0, 1.0, 0.0); // Rotation scene en site
        break;
    case toLUNE_A:
    case toLUNE_C:
    // Rscène avec X vers l'arrière (car RepLune dirigé vers la Lune), Y à gauche et Z vers le haut
//        glRotated(-90.+sitScene, 1.0, 0.0, 0.0); // Ramène Z vertical et le penche d'un angle site
//        glRotated(90.-aziScene, 0.0, 0.0, 1.0); // Ramène X vers l'arrière et fait tourner la scene en azimut.
        break;
//    case SYS_SOL :
    case frSOLEIL :
    case toEARTH_E :
    case toEARTH_c : // Rscène avec X vers l'avant, Y à droite et Z vers le haut
        glRotated(-90.+sitScene, 1.0, 0.0, 0.0); // Ramène Z vertical et le penche d'un angle site
        glRotated(-90.-aziScene, 0.0, 0.0, 1.0); // Ramène X vers l'avant et fait tourner la scene en azimut.
        break;
    }
    glPushMatrix();
}

void TerreGLWidget::paintGL()
{
    if (height1 == 0) return ;

    // Initialisations
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); // On efface ce qu'il y avait avant
    glEnable(GL_LIGHTING);
    glPopMatrix(); // Recupère le repère scène de départ
    glPushMatrix(); // et la sauvegarde pour le prochain appel
    glTranslated(xScene, yScene, zScene); // Pour modif avec touches X, Y, Z et verifier les axes de reference

    // Activation de l'éclairage
    glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE); // On ne traite que les faces externes
    glColorMaterial(GL_FRONT, GL_DIFFUSE);
    glMaterialfv(GL_FRONT, GL_AMBIENT, eteinte);
    glMaterialfv(GL_FRONT, GL_SPECULAR, moyenne);
    glMateriali(GL_FRONT, GL_SHININESS, 32); // Pour limiter les défauts à la limite eclairage/ombre [0-128] en mode specular
    GLfloat pos_lumiere0[4]; // Pour la direction du Soleil (x, y, z, s avec s=0 pour direction et 1 pour position)

    ModePtVue mode = modePtVue;
#if 0
    if (mode == SYS_SOL)
    {
        for (int i=0; i<3; i++) pos_lumiere0[i] = 0.; // Position Soleil
        pos_lumiere0[3] = 1.; // 1. pour positio, 0. pour direction
        glDisable(GL_LIGHTING); // couleur indépendante de l'éclairage
        for (int i = 0; i < 8; i++)
        {
            glColor3dv(colOrb[i]);
            glCallList(dl_Orbs[i]);
        }
        glEnable(GL_LIGHTING); // couleur dépendante de l'éclairage
        return;
    }
#endif
    int nbsat = cine->all_sats->size();
    if (mode == frSATEL && (nbsat <= 0 || !drawSatellites)) mode = toEARTH_c;
    if ((mode == toLUNE_C ||mode == toLUNE_A ) && !drawLune) mode = toEARTH_c;
    double *xyz, mat44[16]; // w[3];

    // On défini la lumière qui vient du Soleil dans Rc
    for (int i=0; i<3; i++) pos_lumiere0[i] = Terre::v3_uTO_c[i]; // Direction Soleil
    pos_lumiere0[3] = 0.; // 1. pour position, 0. pour direction

    switch (mode)
    {
    case frSATEL: // Rscène initial est le repère local satellite
    {
        // Vue depuis le satellite courant qu'on dessine comme un satellite pleiade
        SatGeo *s = cine->all_sats->at(cine->satcou);
        glPushMatrix(); // On sauvegarde le repère scene en cas de destruction par le tracé
        glCallList(dl_Sat);
        glColor3fv(puissante); // Lumière blanche pour empecher la coloration des textures par la dernière couleur du satellite
        glPopMatrix(); // On est de nouveau dans le repère scène
        glTranslated(0., 0., 1.*s->dist); // Translation vers centre Terre
        tm3ToGLmat44(mat44, s->m3_CL); // Convetit et transpose m3_CL en 4x4_LC
        glMultMatrixd(mat44); // On est dans Rc
        break;
    }
    case frZENITH: // Rscène initial est centré Terre
    {
        // On transforme de la base observateur (Z zénit et X vers le sud), en base céleste
        // On pourrait passer d'abord par Greenwich, dessiner la Terre, puis passer dans le repère céleste,
        // mais comme ça on utilise le traitement général qui fait tout à partir du repère céleste.
        //        double m3_CA[9]; m3m3(m3_CA, Terre::m3_CG, Terre::m3_GA);
        tm3ToGLmat44(mat44, Terre::m3_CA); // Convetit et transpose m3_CA en 4x4_AC
        glMultMatrixd(mat44); // On est dans Rc
        break;
    }
    case toLUNE_A:
    {
        glTranslated(0., 0., -gainDist*(Lune::distTL-Lune::dist0));
        // Rscène avec X vers l'arrière (car RepLune dirigé vers la Lune), Y à gauche et Z vers le haut
        glRotated(-90.+sitScene, 1.0, 0.0, 0.0); // Ramène Z vertical et le penche d'un angle site
        glRotated(90.-aziScene, 0.0, 0.0, 1.0); // Ramène X vers l'arrière et fait tourner la scene en azimut.

        tm3ToGLmat44(mat44, Lune::m3_CLa); // Convertit et transpose m3_CL en 4x4_LC
        glMultMatrixd(mat44); // On est dans Rc centré Lune
        // On met la lumière du Soleil
        glLightfv(GL_LIGHT0, GL_POSITION, pos_lumiere0);
        m3ToGLmat44(mat44, Lune::m3_CM); // Convetit m3_CM en 4x4
        glMultMatrixd(mat44); // On est dans Rm (repère globe lunaire) centré Lune
        glCallList(dl_Lune);
        break;
    }
    case toLUNE_C:
    {
        glTranslated(0., 0., -gainDist*(Lune::distTL-Lune::dist0));
        // Rscène avec X vers l'arrière (car RepLune dirigé vers la Lune), Y à gauche et Z vers le haut
        glRotated(-90.+sitScene, 1.0, 0.0, 0.0); // Ramène Z vertical et le penche d'un angle site
        glRotated(90.-aziScene, 0.0, 0.0, 1.0); // Ramène X vers l'arrière et fait tourner la scene en azimut.

        tm3ToGLmat44(mat44, Lune::m3_CLc); // Convertit et transpose m3_CL en 4x4_LC
        glMultMatrixd(mat44); // On est dans Rc centré Lune
        // On met la lumière du Soleil
        glLightfv(GL_LIGHT0, GL_POSITION, pos_lumiere0);
        m3ToGLmat44(mat44, Lune::m3_CM); // Convetit m3_CM en 4x4
        glMultMatrixd(mat44); // On est dans Rm (repère globe lunaire) centré Lune
        glCallList(dl_Lune);
        break;
    }
    case toEARTH_c : // rien a faire, Rc est selon le repère scène avec X vers l'avant et Z vers le haut
        break;
    case frSOLEIL :
        // on se positionne au niveau de la terre
//        glTranslated(-Terre::distTO, 0., 0.);
        // ramène Xsoleil, vers gamma pour passer dans Eclip (puis Zeclip vers ZPolaire pour passer dans céleste)
        glRotated(-Terre::solLongiEclip*180./M_PI, 0., 0., 1.); // ramene OX selon point vernal
        // Attention pas de break ici pour passer dans le repère céleste
    case toEARTH_E : // ramène Zeclip vers Zcéleste (polaire)
        glRotated(-Reperes::epsilon_dg, 1., 0., 0.);
        break;
    }

    if (mode != toLUNE_A && mode != toLUNE_C)
    {
        // On est dans Rc centré Terre. On met la lumière du Soleil
        glLightfv(GL_LIGHT0, GL_POSITION, pos_lumiere0);

        // On dessine la Terre qui est définie dans le repère de Greenwich
        glPushMatrix(); // On sauvegarde Rc, centré Terre
        m3ToGLmat44(mat44, Terre::m3_CG); // Convetit m3_CG en 4x4
        glMultMatrixd(mat44); // On est dans Rg (repère de greenwich)
        glCallList(dl_Terre);
        glPopMatrix(); // On est de nouveau dans Rc, centré Terre
        if (drawLune) // La Lune
        {
            glPushMatrix(); // On sauvegarde Rc, centré Terre
            // On se translate au centre de la Lune
            glTranslated(Lune::v3_TL_c[0],Lune::v3_TL_c[1],Lune::v3_TL_c[2]);
            tm3ToGLmat44(mat44, Lune::m3_CLc); // Convertit et transpose m3_CL en 4x4_LC
            glMultMatrixd(mat44); // On est dans Rc centré Lune
            glCallList(dl_Lune);
            glPopMatrix(); // On est de nouveau dans Rc, centré Terre
        }

        // On dessine des points pour les satellites
        if (drawSatellites)
        {
            glDisable(GL_LIGHTING); // de couleur indépendante de l'éclairage
            for(int k = 0; k < nbsat; k++)
            {
                xyz = cine->all_sats->at(k)->v3_TS_c;
                if (k != cine->satcou) // On trace juste un petit point vert
                {
                    glPointSize(2.f);
                    glColor3f(0.f, 1.f, 0.f);
                    glBegin(GL_POINTS);
                    glVertex3d(xyz[0], xyz[1], xyz[2]);
                    glEnd();

                }
                else if(modePtVue != frSATEL) // Déja tracé en mode satellite
                { // On trace un gros point rouge pour le satellite courant
                    glPointSize(5.f); // Gros point de taille 5
                    glColor3f(1.f, 0.f, 0.f); // et de couleur rouge
                    glBegin(GL_POINTS);
                    glVertex3d(xyz[0], xyz[1], xyz[2]);
                    glEnd();
                }
            }
            glColor3fv(puissante); // Lumière blanche pour empecher la coloration des textures par la dernière couleur du satellite
        }
    }
}


/*----------------------------------------------------------------------------------*/
/* Callback traitement fleches pour modifier le point de vue                        */
/* ^ - V               : +/- angle Site                                             */
/* > - <               : +/- angle Azimut                                           */
/* Home - End          : fenetre vers la gauche - droite                            */
/* Page-Up - Page-Down : fenetre vers le haut, bas                                  */
/* e - E               : eloigne ou approche l'observateur                          */
/* a - A               : effet de grand angle ou zoom                               */
/* Esc                 : point de vue initial                                       */
/* p, P                : Vue avec ou sans perspective  (choix supprimé)             */
/* x,X,y,Y,z,Z : modifie la position de la scene                                    */
/* 0 ou o ou O : Retour à la configuration initiale                                 */
/*----------------------------------------------------------------------------------*/
void TerreGLWidget::keyPressEvent(QKeyEvent *event)
{
    int key = event->key();
    bool ok = false;
    bool majuscule = (Qt::SHIFT == event->modifiers());
    int maj = majuscule ? 1 : 0;
    int smaj = -1 + 2*maj; /* normal = -1; majuscule = +1 */
    double DeltaA = 0.5, DeltaL = distSceneIni/100.;

    switch (key)
    {
    case Qt::Key_Up :
        sitScene += DeltaA ;
        if (sitScene > 180.) sitScene -= 360. ;
        ok = true;
        break ;
    case Qt::Key_Down :
        sitScene -= DeltaA ;
        if (sitScene < -180.) sitScene += 360. ;
        ok = true;
        break ;
    case Qt::Key_Left :
        aziScene -= DeltaA ;
        if (aziScene < -180.) aziScene += 360. ;
        ok = true;
        break ;
    case Qt::Key_Right :
        aziScene += DeltaA ;
        if (aziScene > 180.) aziScene -= 360. ;
        ok = true;
        break ;
    case Qt::Key_Escape : /* Point de vue initial */
        aziScene = aziSceneIni;
        sitScene = sitSceneIni;
        gainDist = 1.;
        gainAngle = 1.;
        xScene = yScene = zScene = 0.;
        ok = true;
        break ;
    case Qt::Key_E : /* gain sur distScene */
        if (majuscule) gainDist *= 0.975;
        else gainDist /= 0.975;
        ok = true;
        break ;
    case Qt::Key_A : /* gain sur angle de vue */
        if (majuscule) gainAngle *= 0.975;
        else gainAngle /= 0.975;
        ok = true;
        break ;
    case Qt::Key_X : /* +/- X */
        xScene -= DeltaL*smaj ;
        ok = true;
        break ;
    case Qt::Key_Y : /* +/- Y */
        yScene -= DeltaL*smaj ;
        ok = true;
        break ;
    case Qt::Key_Z : /* +/- Z */
        zScene -= DeltaL*smaj ;
        ok = true;
        break ;

    case Qt::Key_P : /* Perspective */
        perspective = !maj ;
        ok = true;
        break;

    }
    if (ok)
    {
        setProjection();
        updateGL();
    }
    else QGLWidget::keyPressEvent(event);
}
