#ifndef TWOTLE_H
#define TWOTLE_H
#include <vector>
using std::vector;
/**
 *   Classe permettant de decoder les TLE
 * 
TLE veut dire Two-Line Elements en anglais. Ces deux lignes continennent les
elements orbitaux qui dÃ©finissent l'orbite exacte d'un satellite et la
position du satellite sur son orbite au moment precis appele epoque. L'epoque
des donnees orbitales, ou l'heure exacte a laquelle les donnees presentees sont
valides, est inscrite a la ligne un des TLE. Elle sert a determiner l'age des
elements, donc de savoir si les previsions qui seront generees avec eux seront
precises.
Les TLE d'un grand nombre de satellites sont accessibles sur le site :
http://celestrak.com/NORAD/elements/
 * @author Michel Llibre - ONERA/DCSD.
 */
class TwoLE {
public :
    char name[71];   // mini 71 = 69 caractères + '\n' + '0' de fin
	int catnum; 		/* Numero identifiant le satellite */
	int setnum;			/* Numero bulletin de cette tle (raz a 9999) */
    char designator[9];	/* "AAxxxxxx" = xxxxx numero du lancement dans l'annee AA */
	int year;			/* Annee de la date du releve des donnees */
	double refepoch;	/* Jour fractionnaire de cette date dans l'annee */
	double incl;		/* Inclinaison de l'orbite en degres */
	double raan;		/* Righ Ascension of the Ascending Node (degres) */
	double eccn;		/* Excentricite de l'orbite */
	double argper;		/* Argument du perigee (degres) */
	double meanan;		/* Anomalie moyenne a la date des donnees (degrees) */
	double meanmo;		/* Mouvement moyen en tours/jour */
	double drag;		/* Ralentissement du mouv.moy en tours/jour/jour */
	double nddot6;		/* Derivee 3eme de la revolution en tours/jour/jour/jour */
	double bstar;		/* Terme de trainee ?? */
	int orbitnum;		/* Nombre de revolutions depuis de lancement */
private :
	static unsigned char val[256];
    static int flag_init;
	static void init();

public :
    TwoLE() {}
/**
 *  Construction et decodage d'une TLE
 * @param name		String nom du satellite
 * @param line1		Ligne 1
 * @param line2		Ligne 2
 */
    TwoLE(char * name, char * line1, char * line2);
    void set(char * name, char * line1, char * line2);

/**
 * Construction et decodage d'une TwoLE lue dans un fichier
 * @param nom		Nom du satellite recherche
 * @param filename	Nom d'un fichier contenant des TLE
 * @return          La tle construite ou null si echec
 */
TwoLE(char *nom, char *filename);
/* Renvoie 0 si pas trouvé */
int set(char *nom, char *filename);
	
/**
 * Construction et decodage d'une TwoLE
 * @param nom		Nom du satellite recherche
 * @param filename	Nom d'un fichier contenant des TLE
 * @return          La tle construite ou null si echec
 */
    static TwoLE makeNew(char *nom, char *filename);

/**
 * Lecture des TLE d'un fichier et ajout des TLE lues au Vector tles
 * @param filename	Nom d'un fichier contenant des TLE
 * @param tles          Vecteur de pointeurs sur les tles
 * @return               le nombre de tle ajoutees
 */
    static int readFile(char *filename, vector<TwoLE *> *tles);

/**
 * Verification du format des TLE
 * @param line1
 * @param line2
 * @return 0 si OK, -1 si erreur Checksum, 1 si autre erreur
 */
    static int checkTLE(char * line1, char * line2);

/**
 * Impression d'une TwoLE
 */
    void print();
	
    static void test() ;
};
#endif
