#include "confstarsdialog.h"
#include "ui_confstarsdialog.h"
#include <QMessageBox>
#include <math.h>
#include "starswidget.h"
#include "../cinespace.h"
#include "planets.h"
#include "lune.h"

#define toRadians (M_PI/180.)
#define toDegrees (180./M_PI)


int compare_idxNomCons(const void *e1, const void *e2)
{
    return strcmp(((idxNomCons*) e1)->nom, ((idxNomCons*) e2)->nom);
}

ConfStarsDialog::ConfStarsDialog(StarsWidget *s, QWidget *parent) : QDialog(parent), ui(new Ui::ConfStarsDialog)
{
    ui->setupUi(this);
    setWindowTitle("Configuration observateur");
    notinitok = true;
    stars = s;

    vecCons = new idxNomCons[1 + Constellation::nbConstels];
    strcpy(vecCons[0].nom, "Toutes");
    vecCons[0].idx = 0;
    for (int i = 1 ; i <= Constellation::nbConstels; i++)
    {
        strcpy(vecCons[i].nom, stars->tabCons[i-1].frenchName);
        vecCons[i].idx = i;
    }
    qsort((void *) &vecCons[1], Constellation::nbConstels, sizeof(idxNomCons), compare_idxNomCons);
    for (int i = 0 ; i <= Constellation::nbConstels; i++) ui->comboBox_selConst->addItem(QString::fromUtf8(vecCons[i].nom));

    labelHelpConf = new QLabel(
            tr(
                    "<h4>Configuration visée :</h4>"
                    "<ul><li>Azimut visée : 0 c'est vers le Sud.</li>"
                    "<li>Site visée : 0 pour visée horizontale.</li>"
                    "<li>Champ de vue : 0.01 à 180° (inverse du grossisement).</li>"
                    "<li>Magnitude : Seules les étoiles de cette magnitude<br/>"
                    "ou plus brillante sont affichées.</li></ul>"
                    "<h4>Configuration affichage :</h4>"
                    "<ul><li>Fond ciel BLANC : Etoiles points noirs sur fond blanc.</li>"
                    "<li>Fond ciel NOIR : Etoiles points blancs sur fond noir.</li>"
                    "<li>Fond ciel VARIABLE : Etoiles visibles de nuit seulement.</li>"
                    "<li>Horizon : Si coché, ce qui est sous l'horizon est caché.</li>"
                    "<li>Champ de vue : Si coché, ce qui est hors du champ de vue est caché.</li>"
                    "<li>Nom étoiles : Affiche nom commun des principales étoiles.</li>"
                    "<li>Nom étoiles cons : Affiche nom étoile relatif aux constellations.</li>"
                    "<li>Tracé des constellations : Tracées en pointillés si coché.</li>"
                    "<li>Noms constellations : Si coché, affichage du nom des constellation.</li></ul>"
                    "<h4>k X sol & lune</h4>"
                    "<ul>Coef. multiplicateur de la taille lune et soleil.</ul>"
                    "<h4>Constellations affichées : </h4>"
                    "<ul>Affichage de toutes les constellations ou d'une seule à choisir.</ul>"
                    ), 0);
    labelHelpConf->setWindowTitle("Aide configuration");

    init();

    connect(ui->doubleSpinBox_azi, SIGNAL(valueChanged(double)),this, SLOT(slider_azi_setValue(double)));
    connect(ui->doubleSpinBox_sit, SIGNAL(valueChanged(double)),this, SLOT(slider_sit_setValue(double)));
    connect(ui->doubleSpinBox_alf, SIGNAL(valueChanged(double)),this, SLOT(slider_alf_setValue(double)));
    connect(ui->doubleSpinBox_mag, SIGNAL(valueChanged(double)),this, SLOT(slider_mag_setValue(double)));
    connect(ui->doubleSpinBox_kSL, SIGNAL(valueChanged(double)),this, SLOT(slider_kSL_setValue(double)));

    connect(ui->horizontalSlider_azi, SIGNAL(valueChanged(int)),this, SLOT(spinbox_azi_setValue(int)));
    connect(ui->horizontalSlider_sit, SIGNAL(valueChanged(int)),this, SLOT(spinbox_sit_setValue(int)));
    connect(ui->horizontalSlider_alf, SIGNAL(valueChanged(int)),this, SLOT(spinbox_alf_setValue(int)));
    connect(ui->horizontalSlider_mag, SIGNAL(valueChanged(int)),this, SLOT(spinbox_mag_setValue(int)));
    connect(ui->horizontalSlider_kSL, SIGNAL(valueChanged(int)),this, SLOT(spinbox_kSL_setValue(int)));

    notinitok = false;
}

ConfStarsDialog::~ConfStarsDialog()
{
    labelHelpConf->close();
    delete ui;
    delete vecCons;
}

void ConfStarsDialog::closeEvent(QCloseEvent *event)
{
    labelHelpConf->close();
}


void ConfStarsDialog::init()
{
    ui->doubleSpinBox_azi->setValue(toDegrees * stars->aziVisee);
    ui->horizontalSlider_azi->setValue((int) (toDegrees * stars->aziVisee));

    ui->doubleSpinBox_sit->setValue(toDegrees * stars->sitVisee);
    ui->horizontalSlider_sit->setValue((int) (toDegrees * stars->sitVisee));

    ui->doubleSpinBox_alf->setValue(toDegrees * stars->alfamax);
    ui->horizontalSlider_alf->setValue((int) (toDegrees * stars->alfamax));

    ui->doubleSpinBox_mag->setValue(stars->magmin);
    ui->horizontalSlider_mag->setValue((int) (100. * stars->magmin));

    ui->doubleSpinBox_kSL->setValue(stars->kmulSL);    
    ui->horizontalSlider_kSL->setValue((int) (stars->kmulSL));

    ui->comboBox_selConst->setCurrentIndex(stars->selConstel);

    switch(stars->clarte)
    {
    case NOIR: ui->radioButton_noir->setChecked(true); break;
    case BLANC: ui->radioButton_blanc->setChecked(true); break;
    case GRIS: ui->radioButton_gris->setChecked(true); break;
    }

    ui->checkBox_Horiz->setChecked(stars->flagHoriz);
    ui->checkBox_Champ->setChecked(stars->flagChamp);
    ui->checkBox_drawConstel->setChecked(stars->drawConstel);
    ui->checkBox_sgName->setChecked(stars->drawSgName);
    ui->checkBox_consName->setChecked(stars->drawConsName);
    ui->checkBox_nomStar->setChecked(stars->drawStarName);
    ui->checkBox_Lune->setChecked(stars->drawLune);
    ui->checkBox_Planetes->setChecked(stars->drawPlanetes);

// La lune et les planètes toujours tracées
    ui->checkBox_Lune->setEnabled(false);
    ui->checkBox_Planetes->setEnabled(false);
}

void ConfStarsDialog::on_radioButton_noir_toggled(bool checked)
{
   if (notinitok) return;
   if (checked)
    {
       stars->clarte = NOIR;
       stars->majColors();
       stars->update();
   }
}

void ConfStarsDialog::on_radioButton_blanc_toggled(bool checked)
{
    if (notinitok) return;
    if (checked)
     {
        stars->clarte = BLANC;
        stars->majColors();
        stars->update();
    }
}

void ConfStarsDialog::on_radioButton_gris_toggled(bool checked)
{
    if (notinitok) return;
    if (checked)
     {
        stars->clarte = GRIS;
        stars->majColors();
        stars->update();
    }
}

void ConfStarsDialog::on_checkBox_Horiz_toggled(bool checked)
{
    if (notinitok) return;
    stars->flagHoriz = checked;
    stars->update();
}

void ConfStarsDialog::on_checkBox_Champ_toggled(bool checked)
{
    if (notinitok) return;
    stars->flagChamp = checked;
    stars->update();
}

void ConfStarsDialog::on_checkBox_drawConstel_toggled(bool checked)
{
    if (notinitok) return;
    stars->drawConstel = checked;
    stars->update();
}

void ConfStarsDialog::on_checkBox_Lune_toggled(bool checked)
{
    if (notinitok) return;
    stars->drawLune = checked;
    if (checked) stars->cine->majCine();
}

void ConfStarsDialog::on_checkBox_Planetes_toggled(bool checked)
{
    if (notinitok) return;
    stars->drawPlanetes = checked;
    if (checked) stars->cine->majCine();
}

void ConfStarsDialog::on_checkBox_stellites_toggled(bool checked)
{
    if (notinitok) return;
    stars->drawSatellites = checked;
    if (checked) stars->cine->majCine();
}


void ConfStarsDialog::on_checkBox_sgName_toggled(bool checked)
{
    if (notinitok) return;
    stars->drawSgName = checked;
    stars->update();
}

void ConfStarsDialog::on_checkBox_consName_toggled(bool checked)
{
    if (notinitok) return;
    stars->drawConsName = checked;
    stars->update();
}

void ConfStarsDialog::on_checkBox_nomStar_toggled(bool checked)
{
    if (notinitok) return;
    stars->drawStarName = checked;
    stars->update();
}

void ConfStarsDialog::on_doubleSpinBox_azi_valueChanged(double v)
{
    if (notinitok) return;
    stars->aziVisee = toRadians*v;
    stars->majVisee();
    stars->update();
}

void ConfStarsDialog::on_doubleSpinBox_sit_valueChanged(double v)
{
    if (notinitok) return;
    stars->sitVisee = toRadians*v;
    stars->majVisee();
    stars->update();
}

void ConfStarsDialog::on_doubleSpinBox_alf_valueChanged(double v)
{
    if (notinitok) return;
    stars->alfamax = toRadians*v;
    stars->majVisee();
    stars->update();
}

void ConfStarsDialog::on_doubleSpinBox_mag_valueChanged(double v)
{
    if (notinitok) return;
    stars->magmin = v;
    stars->update();
}

void ConfStarsDialog::on_doubleSpinBox_kSL_valueChanged(double v)
{
    if (notinitok) return;
    stars->kmulSL = v;
    stars->update();
}


////////////////////////////////////
// SLOTS de CONVERSION double -> int
////////////////////////////////////
void  ConfStarsDialog::slider_kSL_setValue(double v)
{ui->horizontalSlider_kSL->setValue((int) rint(v)); }

void  ConfStarsDialog::slider_mag_setValue(double v)
{
    ui->horizontalSlider_mag->setValue((int) rint(v*100.));
}

void  ConfStarsDialog::slider_alf_setValue(double v)
{ui->horizontalSlider_alf->setValue((int) rint(v)); }

void  ConfStarsDialog::slider_sit_setValue(double v)
{ui->horizontalSlider_sit->setValue((int) rint(v)); }

void  ConfStarsDialog::slider_azi_setValue(double v)
{ui->horizontalSlider_azi->setValue((int) rint(v)); }


////////////////////////////////////
// SLOTS de CONVERSION int -> double
////////////////////////////////////
void  ConfStarsDialog::spinbox_kSL_setValue(int v)
{ ui->doubleSpinBox_kSL->setValue((double) v);}

void  ConfStarsDialog::spinbox_mag_setValue(int v)
{
    ui->doubleSpinBox_mag->setValue((double) v/100.);
}

void  ConfStarsDialog::spinbox_sit_setValue(int v)
{ui->doubleSpinBox_sit->setValue((double) v);}

void  ConfStarsDialog::spinbox_azi_setValue(int v)
{ui->doubleSpinBox_azi->setValue((double) v);}

void  ConfStarsDialog::spinbox_alf_setValue(int v)
{
    if (v != 0) ui->doubleSpinBox_alf->setValue((double) v);
    else ui->doubleSpinBox_alf->setValue(0.01);
}

void ConfStarsDialog::on_pushButton_clicked()
{
    labelHelpConf->show();
}

void ConfStarsDialog::on_comboBox_selConst_currentIndexChanged(int index)
{
    if (notinitok) return;
    stars->selConstel = vecCons[index].idx;
    stars->update();
}
