#ifndef ETOILE_H
#define ETOILE_H

class Etoile  {
public:
    const static int nbStars = 9110;
    static int nbGets;  // nombre d'étoiles acquises
    double lastYcorMouv;   // date de dernière correction du mouvement propre (années juliennes/J2000.0)
    static double lastYcorAberr; // date de dernière correction de l'aberrtation (années juliennes/J2000.0)
    bool ok;            // true si données présentes
    int bsNumber;	    // Numéro dans le Bright Star Catalogue
    char bsName[12];	// Nom lu dans le Bright Star Catalogue
    double ra0;		    // Ascension droite a la date J2000.0 dans J2000 en radians
    double ra0t;		// Ascension droite a la date tycorprec dans J2000 en radians
    double ra;		    // Ascension droite a la date tycorprec dans dans repère de la date en radians
    double dec0;		// Déclinaison a la date J2000.0 dans J2000 en radians
    double dec0t;		// Déclinaison a la date tycorprec dans J2000 en radians
    double dec;		    // Déclinaison a la date tycorprec dans repère de la date en radians
    double uCSo[3];     // Direction unitaire étoile a la date tycorMouv dans repère J2000
    double uCS[3];      // Direction unitaire étoile a la date tcou dans repère de la date
    double mag;		    // Magnitude
    int flamsteed;	    // Numéro étoile selon dans la constellation Flamsteed si renseigné
    char bayer[8];	    // Lettre étoile dans la constellation selon Bayer si renseigné
    char constel[4];	// Nom abrégé de la constellation si renseigné
    char sgName[8];     // shortGreekName UTF8 à ma sauce
    char nameEcons[12]; // sgName suivi de constel.
    char nom[16];	    // Nom français de l'étoile si renseigné (UTF8)
    double vRA, vDE;    // vitesse variation ascension et declinaison rd/an a J2000
    double vMod;        // Module vitesse de l'étoile
    // Paramètres utilisés par StaWidget
    bool flagxyOk;      // Vrai x et y dans cone de visée
    bool flagCalc;      // Vrai si x,y calculés
    int ix, iy;

    static Etoile *init(); // A faire avant tout pour charger les fichier étoiles!!!

    Etoile(); // Contructeur initialise qqes champs.

    // Correction mouvement propre de l'étoile à la date yJ (année julienne) après J2000
    void corrMouvement(double yJ);

    // Calcul composantes et coordonnées équatoriales moyennes repère céleste de la date
    void compEquatMoyDate();

    // Calcul coordonnées et composantes équatoriales moyennes repère céleste de la date
    void coordEquatMoyDate();

    // Calcul composantes et coordonnées équatoriales vraies repère céleste de la date
    void compEquatVraiDate();

    // Calcul coordonnées et composantes équatoriales vraies repère céleste de la date
    void coordEquatVraiDate();

    // Calcul coordonnées et composantes équatoriales apparentes repère céleste de la date
    void coordEquatApparent(bool precismax);

    // Calcul composantes et coordonnées équatoriales apparentes repère céleste de la date
    void compEquatApparent(bool precismax);

    /* Mise à jour coordonnees toutes étoiles pour te_j jours juliens après J2000 */
    /* avec correction nutation si corrNut vrai */
    static void misajour(double te_j, double precision, bool corrNut);
};

// Informations sur l'étoile
char *infoStar(char *buf, Etoile *e, double t_y);

class Constellation
{
public:
    Constellation() {}
    static Constellation* init();
    static int nbConstels;      // Nombre de constellations
    char *toNewString();
    int *starsIndex;		// Liste des index (dans tabStars) des étoiles de la constellation
    int *linesOri;		// Liste des index (dans starsIndex) des origines des segments
    int *linesDes;		// Listes des index des destination des segments
    int starsIndexSize;
    int linesSize;
    double uCS[3];              // Direction unitaire Barycentre dans J2000
    char frenchName[40];	// Nom français de la constellation
    char shortName[4];		// Nom court (3 lettres) de la constellation

private:
    double raLabel, decLabel; 	// Barycentre constellation en radians pour le label
};


#endif  // ETOILE_H
