#ifndef LUNE_H
#define LUNE_H

/**
 * Les ephemerides de la lune d'apres Astronomical Algorithms de Jean Meeus (2d Edition).
 * Avec éventuellement correction precession pour passer dans repère écliptique J2000
 * @author Michel Llibre.
 */
#define SIZE_AS 60
// POUR FAIRE UN TEST
void testLune();

// CLASSE STATIQUE. Pas besoinde créer d'instance.

class Lune {
private:
    // Constantes de la classe
    static const double argDa[SIZE_AS];
    static const int argMa[SIZE_AS];
    static const double argMla[SIZE_AS];
    static const double argFa[SIZE_AS];
    static const double sigLa[SIZE_AS];
    static const double sigRa[SIZE_AS];
    static const double argDb[SIZE_AS];
    static const int argMb[SIZE_AS];
    static const double argMlb[SIZE_AS];
    static const double argFb[SIZE_AS];
    static const double sigBb[SIZE_AS];

public:
    static const double rayLune; // rayon de la lune
    static const double dist0;	// Distance Terre-Lune nominale

    // Variables de la classe
    static double distTL;       // Distance Terre-Lune a la date/
    static double longi;        // Longitude
    static double lati;         // Latitude
    static double v3_uTL_e[3];  // Cosinus directeurs vecteur Terre-Lune dans Re
    static double v3_uTL_c[3];  // Cosinus directeurs vecteur Terre-Lune dans Rc
    static double v3_TL_c[3];   // Vecteur Terre-Lune dans Rc
    static double m3_CLc[9];    // Base lune vu de Terre, Y équatorial céleste
    static double m3_CLa[9];    // Base lune vu de Terre, Y hozizontal local
    static double m3_CM[9];     // Base globe lunaire dans Rc (équatorial)
    static double psi;    // angle élongatin Lune-Terre-Soleil
    static double theta;  // angle éclairé vu de la Terre
    static double cosTheta;
    static double inclin; // inclinaison croissant sur la verticale de l'observateur


    static double tcouPos;         // Dernier instant de calcul de la position de la Lune
    static double tcouOri;         // Dernier instant de calcul de l'orientation de la Lune

    // temps en secondes / J2000.
    static void position(double jours, double precision);
    static void orientation(double jours, double precision);
    static void phases();
};

#endif
