#include <QAction>
#include <QImage>
#include <QFileDialog>
#include "starswidget.h"
#include "../mathutil/sp3.h"
#include "terre.h"
#include "lune.h"
#include "planets.h"
#include "../cinespace.h"

StarsWidget::StarsWidget(int w, int h, Etoile *tabEstrellas, Constellation *tabConstellations, CineSpace *p) : QWidget()
{
    cine = p;
    whint = w; hhint = h;
    this->setWindowTitle("Ciel étoilé");

    labelHelp = new QLabel(
                tr(    "<h4>Menu contextuel :</h4>"
                       "<ul><li>Configuration :"
                       "<br/>Affiche une boite de dialogue permettant de modifier ce que l'on observe.</li>"
                       "<li>Sauve Image :"
                       "<br/>Permet de sauvegarder la vue de la fenêtre en image jpeg.</li>"
                       "<li>Help :"
                       "<br/>Affiche cette aide.</li>"
                       "</ul>"
                       ));
    labelHelp->setWindowTitle("Aide Ciel étoilé");

    tabStars = tabEstrellas;
    tabCons = tabConstellations;

    // Quelques init
    gainpix = 1.;
    decalx = decaly = 0;
    selConstel = 0; // Pas de constellation sélectionnée
    initConf();

    majVisee();
    majColors();

    // Les boites de dialogue
    confStarsDialog = new ConfStarsDialog(this, this);

    // On fixe l'arriere plan du cine2dWidget
    setAutoFillBackground(true); // Remplissage automatique de la couleur du fond
    setBackgroundRole(QPalette::Base); // On utilise un fond ordinaire
    setForegroundRole(QPalette::Base);

    // Activation Menu contextuel
    setContextMenuPolicy(Qt::ActionsContextMenu);
    // On ajoute un item au menu pour le ConfStarsDialog
    QAction *confAction = new QAction(tr("&Configuration"), this);
    connect(confAction, SIGNAL(triggered()), this, SLOT(confDialogSlot()));
    addAction(confAction);

    // Le Slot sauver image
    QAction *sauveImg = new QAction(tr("Sauve Image"), this);
    connect(sauveImg, SIGNAL(triggered()), this, SLOT(grabImageSlot()));
    addAction(sauveImg);

    // item "Help"
    QAction *actionHelp = new QAction(tr("&Help"), this);
    connect(actionHelp, SIGNAL(triggered()), this, SLOT(afficheHelpSlot()));
    addAction(actionHelp);

}

// Enregistrement d'image-ecran sous format jpg
void StarsWidget::grabImageSlot()
{
    QImage image(size(), QImage::Format_ARGB32_Premultiplied);
    QPainter pt(&image);
    paintScene(pt);
    QString filename = QFileDialog::getSaveFileName
            (this,tr("Sauve Image"),".",tr("Fichier image (*.jpg)"));
    if (!filename.isEmpty()) image.save(filename);
}

void StarsWidget::initConf()
{
    // Initialisations visée stellaire
    alfamax = 120.*M_PI/180.; // Angle max de la visée (0 à PI)
    aziVisee = 0;  // Azimut de la ligne de visée/au Sud
    sitVisee = 2.*M_PI/180.; // Site de la ligne de visée (0 à PI/2)
    magmin = 3.;
    kmulSL = 1.;

    clarte = GRIS;  // Couleur de fond du cineWidget
    flagHoriz = false;
    flagChamp = false;
    drawStarName = true;
    drawLune = true;
    drawPlanetes = true;
    drawSatellites = false;
    drawConstel = true;
    drawSgName = false;
    drawConsName = true;
}


void StarsWidget::closeEvent(QCloseEvent *event)
{
    if(confStarsDialog) confStarsDialog->close();
    labelHelp->close();
}

// Reception clic menu configuration
void StarsWidget::confDialogSlot()
{
    // Dialogue en modal
    //    confStarsDialog->setModal(true);
    //    if (confStarsDialog->exec() == QDialog::Accepted) confStarsDialog->values();
    // Dialog en modeless
    confStarsDialog->setModal(false);
    confStarsDialog->show();
}

void StarsWidget::majVisee()
{
    double pi2s4 = M_PI*M_PI/4.;
    double sit2 = sitVisee*sitVisee;
    as2 = alfamax/2;
    double a2s4 = as2*as2;
    // Matrice de projection fonction d'azimut et site
    asiTom3(m3_AP, aziVisee, sitVisee);
    // Horizon et champ de vue
    RayHoriz = 0.5*(pi2s4 + sit2)/sitVisee;
    AltHoriz = 0.5*(pi2s4 - sit2)/sitVisee;
    double ymath = -0.5*(pi2s4 - a2s4)/AltHoriz;
    yInterVis = -ymath;
    xInterVis = a2s4 - ymath*ymath;
    if (xInterVis > 0.)
    {
        interVisOk = true;
        xInterVis = sqrt(xInterVis);
        angVis = asin(ymath/as2)*180./M_PI;
        angSol = asin((ymath-AltHoriz)/RayHoriz)*180./M_PI;

    }
    else
    {
        interVisOk = false;
        angSol = 0.;
    }
}


void StarsWidget::majColors()
{
    if (clarte == NOIR)
    {
        couleurCiel.setRgb(255,255,255); // ciel blanc
        couleurStars.setRgb(0, 0,0); // etoiles noires
    }
    else if (clarte == BLANC || clarte == GRIS)
    {
        couleurCiel.setRgb(0, 0,0); // ciel noir
        couleurStars.setRgb(255,255,255); // etoiles blanches
    }
    QPalette pal = palette();
    pal.setColor(QPalette::Base, couleurStars); // ecran couleur étoiles et trou cache couleur ciel
    setPalette(pal);
}

/** Projection direction unitaire/RepèreLocal dans l'écran
  Renvoi les composantes ix, iy dans le repère plan et
  le hauteur au dessus du sol (sinus du site)
*/
bool StarsWidget::uRaToxy(int *ix, int *iy, double *h, int ix0, int iy0, double factor, double *v3_a)
{
    double alpha, rho;
    double v3_p[3];

    *h = v3_a[2];
    // Projection sur le plan orthogonal à la visée
    tm3v3(v3_p, m3_AP, v3_a);
    alpha = acos(-v3_p[2]);
    // Facteur de projection sphérique sur l'écran
    rho = factor*alpha/sqrt(v3_p[0]*v3_p[0]+v3_p[1]*v3_p[1]);
    // Projection sur l'écran de l'étoile
    *ix = ix0 + (int) (rho*v3_p[0]);
    *iy = iy0 - (int) (rho*v3_p[1]);
    //    if (flagHoriz && v3_a[2] < 0.) return false; // Direction sous l'horizon
    if (flagChamp && alpha > as2) return false; // Direction hors du cone de vue
    return true;
}


/** Projection direction unitaire du Soleil/RepèreGreenwich dans l'écran
  Renvoi les composantes ix, iy dans le repère plan et
  le hauteur au dessus du sol (sinus du site)
*/
bool StarsWidget::uSolRgToxy(int *ix, int *iy, double *h, int ix0, int iy0, double factor, double *uvec)
{
    double alpha, rho;
    double v3_a[3], v3_p[3];

    // Projection dans Repère local
    tm3v3(v3_a, Terre::m3_GA, uvec);
    *h = v3_a[2];
    // Projection dans le repère du viseur
    tm3v3(v3_p, m3_AP, v3_a);
    alpha = acos(-v3_p[2]); // angle avec l'axe de visée
    // Facteur de projection sphérique sur l'écran rho = f *
    rho = factor*alpha/sqrt(v3_p[0]*v3_p[0]+v3_p[1]*v3_p[1]);
    // Projection sur l'écran de l'étoile
    *ix = ix0 + (int) (rho*v3_p[0]);
    *iy = iy0 - (int) (rho*v3_p[1]);
    //    if (flagHoriz && v3_a[2] < 0.) return false; // Direction sous l'horizon
    if (flagChamp && alpha > as2) return false; // Direction hors du cone de vue
    return true;
}

/** Projection direction unitaire ou position/Repère Céleste dans l'écran
    Renvoi les composantes ix, iy dans le repère plan et la hauteur au dessus du sol (sinus du site)
    bool proche :
        - false, si "direction unitaire" d'un objet très éloigné non influencé par la position de l'observateur
        - vrai si "position non unitaire" proche pour prise en compte de la localisation de l'observateur sur Terre.
*/
bool StarsWidget::uRcToxy(int *ix, int *iy, double *h, int ix0, int iy0, double factor, double *uvec, bool proche)
{
    double alpha, rho;
    double v3_a[3], v3_g[3], v3_p[3];
    // Projection dans repère Local
    if(proche) // Prise en compte situation observateur
    {
        // D'abord dans Greenwich
        tm3v3(v3_g, Terre::m3_CG, uvec);
        subv3(v3_g, v3_g, Terre::v3_GA); // AL = TL - TA
        unitv3(v3_g, v3_g);
        // Puis dans local
        tm3v3(v3_a, Terre::m3_GA, v3_g);
    }
    // projection directe
    else tm3v3(v3_a, Terre::m3_CA, uvec);

    *h = v3_a[2];
    // Projection sur le plan orthogonal à la visée
    tm3v3(v3_p, m3_AP, v3_a);
    alpha = acos(-v3_p[2]);
    // Facteur de projection sphérique sur l'écran
    rho = factor*alpha/sqrt(v3_p[0]*v3_p[0]+v3_p[1]*v3_p[1]);
    // Projection sur l'écran de l'étoile
    *ix = ix0 + (int) (rho*v3_p[0]);
    *iy = iy0 - (int) (rho*v3_p[1]);
    //    if (flagHoriz && v3_a[2] < 0.) return false; // Direction sous l'horizon
    if (flagChamp && alpha > as2) return false; // Direction hors du cone de vue
    return true;
}

static double ptCardinaux[5][3] = {{1.,0.,0.},{0.,1.,0.},{-1.,0.,0.},{0.,-1.,0.},{0.,0.,1.}};
static const char* const strCardinaux[] = {"S","E","N","O","Z"};

/* Taille pîxels en fonstion de la magnitude */
static double mag2siz(double x) { return 350./((x+13.)*(x+13.)); }


void StarsWidget::paintEvent(QPaintEvent *e)
{
    QPainter painter(this);
    paintScene(painter);
}

void StarsWidget::paintScene(QPainter &painter)
{
    int L = width(), deuxL = 2.*L;
    int H = height(), deuxH = 2.*H;
    //    QPainter painter(this);
    QPen pen, penblue, penblack, pengreen, pentext;
    pen = penblue = penblack = pengreen = pentext = painter.pen();
    penblack.setColor(Qt::black);
    penblue.setColor(Qt::blue);
    pengreen.setColor(Qt::green);
    pentext.setColor(couleurStars);

    int ix0, iy0;
    int ix, iy, ixS, iyS, ixL, iyL;
    double h, uvw[3];

    // Facteur de conversion unités projetées en pixels.
    double f1 = L/alfamax, f2 = H/(as2+(sitVisee < as2 ? sitVisee : as2));
    double factor = gainpix*(f1 < f2 ? f1 : f2);
    double falfax = factor*alfamax;
    double fas2 = factor*as2;
    // Origine au centre de la ligne du bas.
    ix0 = L/2;
    iy0 = H;
    // Décalage pour mettre l'horizon en bas de la fenêtre ou le bord du sitmax
    if (sitVisee < as2) iy0 -= factor*(sitVisee) ;
    else iy0 -= fas2 ;
    // Prise en compte decalage usager
    if (gainpix > 1.02)
    {
        int dxmax = (gainpix-1.)*L/2.;
        int dymax = (gainpix-1.)*H;
        if (decalx > dxmax) decalx = dxmax;
        if (decalx < -dxmax) decalx = -dxmax;
        if (decaly > dymax) decaly = dymax;
        if (decaly < 0) decaly = 0;
        ix0 += decalx;
        iy0 += decaly;
    }

    // Traitement Soleil pour calcul hauteurSol pour la luminosité ambiante
    bool solInCone = false;
    double rsol = kmulSL*Terre::raySol/Terre::distTO;
    double hauteurSol;
    int rxsol = (int) (factor*rsol);
    // Présence/absence Soleil dans cone de visée
//    solInCone = uSolRgToxy(&ixS, &iyS, &hauteurSol, ix0, iy0, factor, Terre::v3_uTO_g);
    solInCone = uRcToxy(&ixS, &iyS, &hauteurSol, ix0, iy0, factor, Terre::v3_uTO_c, false);
    //    hauteurSol += rsol/2;
    // Mise a jour couleur traces si prise en compte hauteur soleil
#define valzero 0.1
    QColor couleurGrise;
    double val;
    if (hauteurSol < 0.) val = valzero*(hauteurSol+1.);
    else if (hauteurSol > valzero) val = (valzero*hauteurSol + 1. - 2.*valzero)/(1.-valzero);
    else val = valzero + (1. - 2.*valzero)*hauteurSol/valzero;
    int r,v,b;
    //        b = 255*val; r = b/2; v = 3*b/4;
    r = v = b = 255*val;
    couleurGrise.setRgb(r, v, b);
    if (clarte == GRIS) couleurCiel = couleurGrise;


    // Mise en place du cache de la couleur des tracés, l'intérieur étant couleur inverse
    // ce qui permet de voir les tracés à l'intérieur
    QPainterPath pathVisee;
    if (flagChamp && flagHoriz)
    {
        if (interVisOk) // existe intersection
        {
            pathVisee.moveTo(ix0 + factor*xInterVis, iy0 + factor*yInterVis);
            pathVisee.arcTo(ix0 - fas2, iy0-fas2, falfax, falfax, angVis, 180. - 2.*angVis);
            pathVisee.arcTo(ix0-factor*RayHoriz, iy0-factor*(AltHoriz + RayHoriz), 2.*factor*RayHoriz, 2.*factor*RayHoriz, 180. - angSol, 180. + 2.*angSol);
        }
        else // Si pas d'intersection, seul le Champ est pris en compte
        {
            pathVisee.addEllipse(ix0-fas2, iy0-fas2, falfax, falfax);
        }
    }
    else if (flagChamp) pathVisee.addEllipse(ix0-fas2, iy0-fas2, falfax, falfax);
    else if (flagHoriz) pathVisee.addEllipse(ix0-factor*RayHoriz, iy0-factor*(AltHoriz + RayHoriz), 2.*factor*RayHoriz, 2.*factor*RayHoriz);
    else pathVisee.addRect(0.,0.,L,H);
    painter.setPen(Qt::NoPen); // Pour masquer les contours des caches
    painter.setBrush(QBrush(couleurCiel, Qt::SolidPattern));
    painter.drawPath(pathVisee);

    // Utilisation d'un pen pour pouvoir fixer la taille des points, car pas de painter.setWidth()
    painter.setPen(Qt::SolidLine); // pour rétablir le tracé des contours
    pen = painter.pen();
    pen.setCapStyle(Qt::RoundCap);
    pen.setColor(couleurStars);
    painter.setBrush(Qt::NoBrush);

    //Dessin ligne de l'horizon
    if (!flagHoriz)
    {
        penblue.setDashPattern(QVector<qreal>() << 3 << 3);
        penblue.setStyle(Qt::CustomDashLine);
        painter.setPen(penblue);
        //painter.drawArc(ix0-factor*RayHoriz, iy0-factor*(AltHoriz + RayHoriz), 2.*factor*RayHoriz, 2.*factor*RayHoriz, (int) ((180. - angSol)*16), (int) ((180. + 2.*angSol)*16));
        painter.drawArc(ix0-factor*RayHoriz, iy0-factor*(AltHoriz + RayHoriz), 2.*factor*RayHoriz, 2.*factor*RayHoriz, 180*16, 180*16);
    }

    painter.setPen(pen);

    // Dessin des points cardinaux
    for(int npt = 0; npt < 5; npt++)
        if (uRaToxy(&ix, &iy, &h, ix0, iy0, factor, ptCardinaux[npt]))
        {
            painter.drawLine(ix-5, iy, ix+5, iy);
            painter.drawLine(ix, iy-5, ix, iy+5);
            painter.drawText(ix+5, iy-5, strCardinaux[npt]);
        }

    // Dessin du soleil
    if (solInCone)
    {
        QColor soleil; soleil.setRgb(128,0,0);
        painter.setBrush(QBrush(soleil, Qt::SolidPattern));
        painter.drawEllipse(ixS-rxsol, iyS-rxsol, 2*rxsol, 2*rxsol);
        if (clarte == GRIS) painter.setPen(penblack);
        painter.drawText(ixS+rxsol, iyS+rxsol-5, "SOLEIL");
        painter.setPen(pen);
    }
    if (drawLune)
    {
        // Dessin de la Lune
        bool luneInCone = false;
        double rLune = kmulSL*Lune::rayLune/Lune::distTL;
        double hauteurLune;
        int rxLune = (int) (factor*rLune);
        // Présence/absence Lune dans cone de visée
        luneInCone = uRcToxy(&ixL, &iyL, &hauteurLune, ix0, iy0, factor, Lune::v3_TL_c, true);

        if (luneInCone)
        {
            painter.setBrush(QBrush(Qt::gray, Qt::SolidPattern));
            painter.drawEllipse(ixL-rxLune, iyL-rxLune, 2*rxLune, 2*rxLune);
            if (clarte == GRIS && hauteurSol > 0.) painter.setPen(penblack);
            painter.drawText(ixL+rxLune, iyL+rxLune-5, "LUNE");
            painter.setPen(pen);
        }
    }

    // Dessin des étoiles et satellites systématique si clarte == BLANC ou NOIR
    // Si clarté == GRIS les étoiles et les satellites ne sont vus que si le Soleil est couché
    if (clarte != GRIS || hauteurSol < 0.) // Saute si variable et il fait pour
    {
        Etoile *e;
        // Dessin d'un point pour les étoiles
        for (int i = 1; i <= Etoile::nbStars; i++)
        {
            e = &tabStars[i];
            if (!e->ok) continue;
            e->flagxyOk = false;
            e->flagCalc = false;
            if (e->mag > magmin) continue;
            if (selConstel > 0)
            {
                e->flagCalc = true;
                if(0 != strcmp(e->constel, tabCons[selConstel-1].shortName)) continue;
            }
            if (!(e->flagxyOk = uRcToxy(&e->ix, &e->iy, &h, ix0, iy0, factor, e->uCS, false))) continue;
            pen.setWidthF(mag2siz(e->mag));
            painter.setPen(pen);
            painter.drawPoint(e->ix, e->iy);
            if (drawStarName && !drawSgName && strlen(e->nom) > 0)
                painter.drawText(e->ix, e->iy-5, QString::fromUtf8(e->nom));
        }
        pen.setWidth(1); // retour largeur standard

        // Dessin des constellations
        double memAlfamax = alfamax; alfamax *= 1.05;
        pengreen.setDashPattern(QVector<qreal>() << 3 << 3);
        pengreen.setStyle(Qt::CustomDashLine);

        Constellation *constel;
        for (int i = 0 ; i < Constellation::nbConstels; i++)
        {
            if (selConstel > 0 && i != selConstel-1) continue;

            int ix, iy;
            constel = &tabCons[i];
            Etoile *eori, *edes, *e;
            if (drawConstel)
            {
                painter.setPen(pengreen);
                bool intori, intdes;
                for (int i=0; i < constel->linesSize; i++)
                {
                    // pas de tracé si etoiles mal décodées
                    eori = &tabStars[constel->starsIndex[constel->linesOri[i]]]; if(!eori->ok) continue;
                    edes = &tabStars[constel->starsIndex[constel->linesDes[i]]]; if(!edes->ok) continue;
                    // pas de tracé si étoile hors vue
                    if (!eori->flagCalc) eori->flagxyOk = uRcToxy(&eori->ix, &eori->iy, &h, ix0, iy0, factor, eori->uCS, false);
                    if (!edes->flagCalc) edes->flagxyOk = uRcToxy(&edes->ix, &edes->iy, &h, ix0, iy0, factor, edes->uCS, false);
                    // Si aucun des deux visibles pas de tracé
                    if (!eori->flagxyOk && !edes->flagxyOk) continue;
                    intori = intdes = true;
                    if (eori->ix < 0 || eori->ix > L || eori->iy < 0 || eori->iy > H) intori = false;
                    if (edes->ix < 0 || edes->ix > L || edes->iy < 0 || edes->iy > H) intdes = false;
                    if (intori || intdes)
                        painter.drawLine(eori->ix,eori->iy, edes->ix, edes->iy);
                }
            }
            // Ecriture des noms abrégés (lettre grecque) des étoiles de la constellation
            if (drawSgName)
            {
                painter.setPen(pentext);
                for (int i = 0 ; i < constel->starsIndexSize; i++)
                {
                    e = &tabStars[constel->starsIndex[i]];
                    if(!e->ok) continue;
                    if (!e->flagxyOk && !(e->flagxyOk = uRcToxy(&e->ix, &e->iy, &h, ix0, iy0, factor, e->uCS, false))) continue;
                    if (strlen(e->nom) > 0) painter.drawText(e->ix, e->iy-5, QString::fromUtf8(e->nom));
                    else painter.drawText(e->ix, e->iy-5, QString::fromUtf8(e->sgName));
                }
            }
            // Ecriture du nom français de la constellation
            if (drawConsName && uRcToxy(&ix, &iy, &h, ix0, iy0, factor, constel->uCS, false))
            {
                QString nom(QString::fromUtf8(constel->frenchName));
                painter.setPen(penblue);
                int w = painter.fontMetrics().width(nom);
                painter.drawText(ix-w/2, iy, nom);
            }
        }
        alfamax = memAlfamax;

        // Dessin des satellites
        if (drawSatellites)
        {
            painter.setPen(pen); // taille 1
            for (int i = 0; i < cine->all_sats->size(); i++)
            {
                SatGeo *s = cine->all_sats->at(i);
                if(!uRcToxy(&ix, &iy, &h, ix0, iy0, factor, s->v3_TS_c, true)) continue;
                painter.drawPoint(ix, iy);
                painter.drawText(ix, iy-5, s->nom);
            }
        }
    }

    if(drawPlanetes)
    {
        // Tracé des planètes
        for (int i = 0; i < 8; i++)
        {
            if (i == 2) continue;
            //           if (Planets::magpla[i] > magmin) continue;
            if (!uRcToxy(&ix, &iy, &h, ix0, iy0, factor, unitv3(uvw, Planets::v8x3_TP_c[i]), false)) continue;
            pen.setWidthF(mag2siz(Planets::magpla[i]));
            painter.setPen(pen);
            painter.drawPoint(ix, iy);
            painter.drawText(ix, iy-5, Planets::noms[i]);
        }
        pen.setWidth(1); // retour largeur standard
    }

    if(drawLune) // Trace du quartier de Lune dans le coin haut-gauche
    {
        // Un carré noir en haut à gauche pour dessiner le croissant de Lune
        double box = 50, bs2 = box/2.;
        painter.setPen(Qt::NoPen);
//        painter.setBrush(Qt::black);
        painter.setBrush(QBrush(couleurGrise, Qt::SolidPattern));
        painter.drawRect(0,0,box, box);
        // Croissant de Lune dessiné en blanc à bords rouge
        pen.setColor(Qt::red);
        painter.setPen(pen);
        painter.setBrush(Qt::white);
        int rxLune = box*0.4, ryLune = rxLune;
        QPainterPath pathLune;
        double raLune = Lune::cosTheta*rxLune;
        if (Lune::theta < 0.)
        {
            rxLune = -rxLune;
            raLune = -raLune;
        }
        pathLune.moveTo(0, ryLune);
        pathLune.arcTo(-rxLune, -ryLune, 2*rxLune, 2*ryLune, -90, 180);
        pathLune.arcTo(-raLune, -ryLune, 2*raLune, 2*ryLune, 90, -180);

        painter.translate(bs2, bs2);
        painter.rotate(-Lune::inclin*180./M_PI); // rotate dans le sens retrograde
        painter.drawPath(pathLune);
        painter.resetTransform();
    }

    // Tracé d'un réticule précisant la direction de visée qui n'est pas forcément centrale
    // à cause du déplacement prenant en compte l'horizon
    pen.setDashPattern(QVector<qreal>() << 1 << 8);
    pen.setStyle(Qt::CustomDashLine);
    pen.setColor(Qt::red);
    painter.setPen(pen);
    painter.drawLine(ix0,0, ix0, H);
    painter.drawLine(0,iy0, L, iy0);

}

QSize StarsWidget::sizeHint() const
{
    return QSize( whint, hhint );
}

//// Mise a jour longitude latitude
//void StarsWidget::majLongiLati(double longi, double lati)
//{
//    // Matrice d'attitude du repère local
//    longitude = longi;
//    latitude = lati;
//    lolaAstroTom3(m3_GA, longitude, latitude);
//    Terre::cgeod2ccart(v3_GA,longitude, latitude,0.);
//}
