#ifndef TERRE_H
#define TERRE_H

// POUR FAIRE UN TEST
void testTerre();


// Structure utilisee pour stocker les les heures et azimuts de lever et coucher du soleil
typedef struct {
    double leverLoc;  // instant lever local en jours dec. par rapport à minuit
    double coucherLoc; // instant coucher local local en jours dec. par rapport à minuit
    double leverUT;  // date UT lever jours dec. /J2000
    double coucherUT; // date UT lever jours dec. /J2000
    double azilever; // sens trigo +, 0 au Sud
    double azicoucher; // sens trigo +, 0 au Sud
} tsunhzero;

// CLASSE STATIQUE. Pas besoinde créer d'instance.
// Attention les coordonnées du Soleil sont relative à des repères moyens de la date

class Terre {
public:
    /* ------------------------
       Constantes de la classe
     * ------------------------ */
    static const double siecleJul_j;
    static const double siecleJul_s;

    /** Rayon du Soleil **/
    static const double raySol;

    /** Unite astronomique (valeur IERS 2003) en m */
    static const double UA;

    /** excentricite nominale de l'orbite Terre */
    static const double e0;


    /** excentricite prise encompte de l'orbite Terre non nulle si flagcpx >= 1 */
    static double e;

    /** Annee siderale en jours */
    static const double anside2j;

    /** Heure siderale Greenwich a J2000.0 en degres */
    static const double hsg0_dg;

    /** Heure siderale Greenwich a J2000.0 en radians */
    static const double hsg0;

    /** Heure siderale Greenwich a J2000.0 en secondes (67310.54840) */
    static const double hsg0_s;

    /** Annee tropique en jours (differente Annee siderale si flagcpx >= 1) */
    static double antrop2j;

    /** Annee tropique en secondes (differente Annee siderale si flagcpx >= 1) */
    static double antrop2s;

    /** Vitesse de rotation de la Terre en rd/s */
    static const double omegaT;

    /** Rayon equatorial Terre en m (valeur IERS 2003) */
    static const double rT;

    /** Cte. gravitation de la Terre en S.I. (valeur IERS 2003) */
    static const double GM;

    /** Harmonique spherique J2 du potentiel Terrestre (valeur IERS 2003) */
    static const double J2;

    /** Harmonique spherique J3 du potentiel Terrestre */
    static const double J3;

    /**    */
    static const double r2J2;

    /** Aplatissement de l'ellipsoide terrestre (valeur IERS 2003) pris nul si flagcpx < 2 */
    static double f;

    /** Parametre p de l'ellipsoide terrestre */
    static double pT;

    /** Carre de l'excentricite de l'ellipsoide terrestre */
    static double eT2;

    /* -------------------------------------------------------
 Variables de la classe Terre mises a jour par Terre.orientation(t)
 concernent la rotation de la Terre sur elle-meme
     * ------------------------------------------------------- */
    /* Dernier instant de calcul de l'orientation de la Terre */
    static double tjOrien;

    /** Heure siderale Greenwich a la date t en radians */
    static double hsg, coshsg, sinhsg;

    /** Matrice passage Rc -> Rg */
    static double m3_CG[9];

    /* -------------------------------------------------------
Variables de la classe Terre mise a jour par Terre.possoleil(t)
concernent la révolution de la Terre autour du Soleil
     * ------------------------------------------------------- */
    /** Dernier instant de calcul de la position de la Terre */
    static double tjSol;

    /** Distance du Soleil a la Terre */
    static double distTO;

    /** Cosinus directeurs vecteur Terre-Soleil dans Rc */
    static double v3_uTO_c[3];

//    /** Cosinus directeurs vecteur Terre-Soleil dans Rg */
//    static double v3_uTO_g[3];

    /** Ascension droite du Soleil dans Rc */
    static double solAscen;

    /** Declinaison du Soleil dans Rc */
    static double solDecli;

    /** Longitude ecliptique du Soleil */
    static double solLongiEclip;
    static double sinLE, cosLE;
    static double solLongiPeri; // Pour calcul précis aberration annuelle

//    /** Longitude du Soleil dans Rg */
//    static double solLongi;

    /** Latitude du Soleil dans Rg */
    static double solLati;

    /** Un observateur sur Terre */
    static double longobs, latobs;
    static double m3_GA[9], m3_CA[9], v3_GA[3];

    /** Orientation de la Terre dans le repère moyens de la date
          @param tu_j 	Instant du calcul TU en jours / J2000
          @param prec 	précision désirée pour les résultats
    */
    static void orientation(double tu_j, double prec);

    /** Position du Soleil
          @param te_j 	Instant du calcul TE en jours / J2000
          @param prec 	précision désirée pour les résultats
    */
    static void posSoleil(double te_j, double precision);

    /** Situation observateur */
    // Mise a jour longitude latitude
    static void setLongiLati(double longi, double lati);


    /** Lever et coucher du soleil, heure et azimut
         @param jours 	Instant du calcul en jours / J2000 (TU)
         @param tephem 	Instant du calcul en jours / J2000 (TT)
         @param longiA  Longitude du lieu d'observation
         @param latiA   Latitude du lieu d'observation
         @param pt      pointeur struct resultat
    */
    static void sunhzero(tsunhzero *pt, double jours, double tephem, double longiA, double latiA, int lnc);


    /** Conversion coordonnees geodesiques -> cartesiennes
          @param V 		Sortie V[3] : x,y,z
          @param longi 		longitude
          @param lati 		latitude geodesique
          @param alti 		altitude geodesique
    */
    static void cgeod2ccart(double *V, double longi, double lati, double alti);

    /** Conversion coordonnees cartesiennes -> geodesiques
          @param V 	  Sortie V[0:2] : longi, lati, h
          @param M	  double [3] des coordonnees cartesiennes x,y,z
          @param eps   precision relative desiree
    */
    static void ccart2cgeod(double *V, double *M, double eps);

    /** Impression position Soleil dans Rc .
    */
    static void print();
};

#endif // TERRE_H
