#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <float.h>
#ifndef M_PI
#define M_PI	3.14159265358979323846
#endif
#include "sp3.h"

static int printformat=1113;  // 1113 -> 11.3g     0113 -> 11.3f     
static char strfmt[10]=" %11.3lg";
/**********************************************************************/
/*           UTILITAIRES MATRICIELS DIMENSION 3                       */
/**********************************************************************/
/*---------------------------------------------------------------------
	toString d'un vecteur de R3
	-----------------------------

	Entrées :
		char *sbuf		: Buffer pour le string
		double *a	: Adresse du v3 imprimé sur la ligne
		int n       : nombre de signes en tout
		int d       : nombre de chiffres après la virgule
----------------------------------------------------------------------*/
char* sprintv3(char *sbuf, double *a, int n, int d)
{
        int i, m;
	char *s = sbuf;
	sprintf(strfmt+2,"%d.%dlf",n,d);
	for (i = 0 ; i < 3 ; i++) {m = sprintf(s, strfmt,a[i]) ; s += m;}
	return sbuf;
}

/*---------------------------------------------------------------------
	Impression d'un vecteur de R3
	-----------------------------

	Entrées :
		char *s		: Texte à imprimer (pas de Rc ajouté à la fin).
		double *a	: Adresse du v3 imprimé sur la ligne
----------------------------------------------------------------------*/
void printv3(char *s, double *a)
{
int i;

	sprintf(strfmt+2,"%d.%dl%c",(printformat/10)%100,(printformat%10),
		printformat/1000 ? 'g' : 'f');

	printf("%s",s) ;
	for (i = 0 ; i < 3 ; i++) printf(strfmt,a[i]) ;
	printf("\n");
}

/* Idem en spécifiant fichier de sortie et format pour les nombres 
   genre : "%11.3lf" ou "%11.3lg" */
void fprintv3(char *s, double *a, FILE *fich, char *format)
{
int i;

	fprintf(fich, "%s",s) ;
	for (i = 0 ; i < 3 ; i++) fprintf(fich, format, a[i]) ;
	fprintf(fich, "\n");
}




/*---------------------------------------------------------------------
	Affection d'un scalaire aux composantes d'un vecteur de R3
	----------------------------------------------------------

	Entrees:
		double *u	: Adresse du v3 affecté
		double s    : Scalaire à affecter

	Sorties:
		u est affecté
		Retour		: Adresse du v3 affecté

----------------------------------------------------------------------*/
double *setv3(double* u, double s)
{
    u[0] = s ;
    u[1] = s ;
    u[2] = s ;
	return u;
}

/*---------------------------------------------------------------------
	Affection des composantes à un vecteur de R3
	--------------------------------------------

	Entrees:
		double *u	: Adresse du v3 affecté
		double x,y,z: Composantes à affecter

	Sorties:
		u est affecté
		Retour		: Adresse du v3 affecté

----------------------------------------------------------------------*/
double *iniv3(double *v, double x, double y, double z)
{ v[0] = x; v[1] = y; v[2] = z; return v;}

/*---------------------------------------------------------------------
	Recopie d'un vecteur de R3 : U = V
	----------------------------------

	Entrees:
		double *u	: Adresse du v3 affecté
		double *v	: Adresse vecteur donné

	Sorties:
		u est affecté
		Retour		: Adresse du v3 affecté
----------------------------------------------------------------------*/
double *cpyv3(double* u, double* v)
{
    u[0] = v[0] ;
    u[1] = v[1] ;
    u[2] = v[2] ;
	return u;
}

/*---------------------------------------------------------------------
	Négation d'un vecteurs de R3 : U = -V
	-------------------------------------

	Entrees:
		double *u	: Adresse du v3 affecté
		double *v	: Adresse vecteur donné

	Sorties:
		u est affecté
		Retour		: Adresse du v3 affecté

    Remarque : u et v peuvent occuper la même mémoire.
----------------------------------------------------------------------*/
double *negv3(double* u, double* v)
{
    u[0] = -v[0] ;
    u[1] = -v[1] ;
    u[2] = -v[2] ;
	return u;
}
/*---------------------------------------------------------------------
    Valeur absolue d'un vecteurs de R3 : U = |V|
	--------------------------------------------

	Entrees:
		double *u	: Adresse du v3 affecté
		double *v	: Adresse vecteur donné

	Sorties:
		u est affecté
		Retour		: Adresse du v3 affecté

    Remarque : u et v peuvent occuper la même mémoire.
----------------------------------------------------------------------*/
double *absv3(double* u, double* v)
{
    u[0] = fabs(v[0]) ;
    u[1] = fabs(v[1]) ;
    u[2] = fabs(v[2]) ;
	return u;
}
/*---------------------------------------------------------------------
	Composante maximale d'un vecteur de R3 : s = max(Ui)
	----------------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: Composante maximale du vecteur
----------------------------------------------------------------------*/
double xmaxv3(double* u)
{
	double s = u[0];
	if (u[1]>s)s=u[1];
	if (u[2]>s)s=u[2];
	return s;
}

/*---------------------------------------------------------------------
	Composante max. en val. abs. d'un vecteur de R3 : s = max(|Ui|)
	---------------------------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: Composante maximale du vecteur, en val. abs.
----------------------------------------------------------------------*/
double xamaxv3(double* u)
{
	double s = fabs(u[0]);
	if (fabs(u[1])>s)s=fabs(u[1]);
	if (fabs(u[2])>s)s=fabs(u[2]);
	return s;
}
/*---------------------------------------------------------------------
	Index de la composante max d'un vecteur de R3
	---------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: index de la composante maximale du vecteur
----------------------------------------------------------------------*/
int imaxv3(double* u)
{
	int i = 0 ;
	double s = u[0];
	if (u[1]>s) { s=u[1]; i = 1;}
	if (u[2]>s) i = 2;
	return i;
}

/*---------------------------------------------------------------------
	Index de la composante max en v.abs. d'un vecteur de R3
	-------------------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: index de la composante maximale en v.a.
----------------------------------------------------------------------*/
int iamaxv3(double* u)
{
	int i = 0 ;
	double s = fabs(u[0]);
	if (fabs(u[1])>s) { s=fabs(u[1]); i = 1;}
	if (fabs(u[2])>s) i = 2;
	return i;
}

/*---------------------------------------------------------------------
	Composante minimale d'un vecteurs de R3 : s = min(Ui)
	-----------------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: Composante minimale du vecteur
----------------------------------------------------------------------*/
double xminv3(double* u)
{
	double s = u[0];
	if (u[1]<s)s=u[1];
	if (u[2]<s)s=u[2];
	return s;
}

/*---------------------------------------------------------------------
	Composante min. en val. abs. d'un vecteur de R3 : s = min(|Ui|)
	---------------------------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: Composante minimale du vecteur, en val. abs.
----------------------------------------------------------------------*/
double xaminv3(double* u)
{
	double s = fabs(u[0]);
	if (fabs(u[1])<s)s=fabs(u[1]);
	if (fabs(u[2])<s)s=fabs(u[2]);
	return s;
}

/*---------------------------------------------------------------------
	Index de la composante min d'un vecteur de R3
	---------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: index de la composante maximale du vecteur
----------------------------------------------------------------------*/
int iminv3(double* u)
{
	int i = 0 ;
	double s = u[0];
	if (u[1]<s) { s=u[1]; i = 1;}
	if (u[2]<s) i = 2;
	return i;
}

/*---------------------------------------------------------------------
	Index de la composante min en v.abs. d'un vecteur de R3
	-------------------------------------------------------

	Entrees:
		double *u	: Adresse vecteur donné

	Sorties:
		Retour		: index de la composante maximale en v.a.
----------------------------------------------------------------------*/
int iaminv3(double* u)
{
	int i = 0 ;
	double s = fabs(u[0]);
	if (fabs(u[1])<s) { s=fabs(u[1]); i = 1;}
	if (fabs(u[2])<s) i = 2;
	return i;
}


/*---------------------------------------------------------------------
	Multiplication d'un vecteur de R3 par un scalaire : U = sV
	----------------------------------------------------------

	Entrees:
		double *U	: Adresse du vecteur affecté
		double *V	: Adresse du vecteur de R3 donné
		double s	: Scalaire donné

	Sorties:
		U est affecté
		Retour		: Adresse du vecteur affecté

	Remarque : U peut occuper la même mémoire que V
----------------------------------------------------------------------*/
double *scav3(double* u, double s, double* v)
{
    u[0] = s * v[0] ;
    u[1] = s * v[1] ;
    u[2] = s * v[2] ;
	return u;
}

/*---------------------------------------------------------------------
	Addition de vecteurs de R3 : U = V + W
	--------------------------------------

	Entrees:
		double *U	: Adresse du vecteur affecté
		double *V	: Adresse du premier vecteur de R3 donné
		double *W	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		U est affecté
		Retour		: Adresse du vecteur affecté

	Remarque : U peut occuper la même mémoire que V ou W
----------------------------------------------------------------------*/
double *addv3(double* u, double* v, double* w)
{
    u[0] = v[0] + w[0] ;
    u[1] = v[1] + w[1] ;
    u[2] = v[2] + w[2] ;
	return u;
}

/*---------------------------------------------------------------------
	Soustraction de vecteurs de R3 : U = V - W
	------------------------------------------

	Entrees:
		double *U	: Adresse du vecteur affecté
		double *V	: Adresse du premier vecteur de R3 donné
		double *W	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		U est affecté
		Retour		: Adresse du vecteur affecté

	Remarque : U peut occuper la même mémoire que V ou W
----------------------------------------------------------------------*/
double *subv3(double* u, double* v, double* w)
{
    u[0] = v[0] - w[0] ;
    u[1] = v[1] - w[1] ;
    u[2] = v[2] - w[2] ;
	return u;
}

/*---------------------------------------------------------------------
	Multiplication terme a terme de vecteurs de R3 : U = V x W
	----------------------------------------------------------

	Entrees:
		double *U	: Adresse du vecteur affecté
		double *V	: Adresse du premier vecteur de R3 donné
		double *W	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		U est affecté
		Retour		: Adresse du vecteur affecté

	Remarque : U peut occuper la même mémoire que V ou W
----------------------------------------------------------------------*/
double *mulv3(double* u, double* v, double* w)
{
    u[0] = v[0] * w[0] ;
    u[1] = v[1] * w[1] ;
    u[2] = v[2] * w[2] ;
	return u;
}

/*---------------------------------------------------------------------
	Division terme a terme de vecteurs de R3 : U = V / W
	----------------------------------------------------

	Entrees:
		double *U	: Adresse du vecteur affecté
		double *V	: Adresse du premier vecteur de R3 donné
		double *W	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		U est affecté
		Retour		: Adresse du vecteur affecté
----------------------------------------------------------------------*/
double *divv3(double* u, double* v, double* w)
{
    u[0] = v[0] / w[0] ;
    u[1] = v[1] / w[1] ;
    u[2] = v[2] / w[2] ;
	return u;
}
/*---------------------------------------------------------------------
	Cumulation de vecteurs de R3 : U = V + aW
	-----------------------------------------

	Entrees:
		double *U	: Adresse du vecteur affecté
		double *V	: Adresse du premier vecteur de R3 donné
		double a	: Scalaire donné
		double *W	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		U est affecté
		Retour		: Adresse du vecteur affecté

	Remarque : U peut occuper la même mémoire que V ou W
----------------------------------------------------------------------*/
double *cumv3(double* u, double* v,double a, double* w)
{
    u[0] = v[0] + a*w[0] ;
    u[1] = v[1] + a*w[1] ;
    u[2] = v[2] + a*w[2] ;
	return u;
}

/*---------------------------------------------------------------------
	Produit scalaire de vecteurs de R3 : s = <A . B>
	------------------------------------------------

	Entrees:
		double *A	: Adresse du premier vecteur de R3 donné
		double *B	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		Retour		: produit scalaire
----------------------------------------------------------------------*/
double dotv3(double* a, double* b)
{
    return(a[0]*b[0]+a[1]*b[1]+a[2]*b[2]);
}

/*---------------------------------------------------------------------
	Produit tensoriel de vecteurs de R3 : M = V1 * V2'
	--------------------------------------------------

	Entrees:
		double *M	: Adresse de la matrice de R3 affectée
		double *V1	: Adresse du premier vecteur de R3 donné
		double *V2	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		M est affectée
		Retour		: Adresse de la matrice de R3 affectée
----------------------------------------------------------------------*/
double *tensv3 (double *mato, double *v1, double *v2)
{
    mato[0] = v1[0] * v2[0] ;
    mato[1] = v1[1] * v2[0] ;
    mato[2] = v1[2] * v2[0] ;
    mato[3] = mato[1] ;
    mato[4] = v1[1] * v2[1] ;
    mato[5] = v1[2] * v2[1] ;
    mato[6] = mato[2] ;
    mato[7] = mato[5] ;
    mato[8] = v1[2] * v2[2] ;
	return mato;
}
/*---------------------------------------------------------------------
	Produit vectoriel de vecteurs de R3 : A = B ^ C
	-----------------------------------------------

	Entrees:
		double *A	: Adresse du vecteur affecté
		double *B	: Adresse du premier vecteur de R3 donné
		double *C	: Adresse du deuxième vecteur de R3 donné

	Sorties:
		A est affecté
		Retour		: Adresse du vecteur affecté

	Remarque : A peut occuper la même mémoire que B ou C
----------------------------------------------------------------------*/
double *crossv3(double* a, double* b, double* c)
{
double p, q ;
      p  = b[1] * c[2] - b[2] * c[1];
      q  = b[2] * c[0] - b[0] * c[2];
    a[2] = b[0] * c[1] - b[1] * c[0];
    a[0] = p ;
    a[1] = q ;
	return a;
}

/*---------------------------------------------------------------------
	Somme des composantes d'un vecteurs de R3 : s = Sigma Ui
	--------------------------------------------------------

	Entrees:
		double *A	: Adresse vecteur donné

	Sorties:
		Retour		: Somme des composantes
----------------------------------------------------------------------*/
double xsumv3(double* a)
{
    return(a[0]+a[1]+a[2]);
}

/*---------------------------------------------------------------------
	Carré d'un vecteurs de R3 : s = A'*A
	------------------------------------

	Entrees:
		double *A	: Adresse vecteur donné

	Sorties:
		Retour		: Somme des carrés des composantes
----------------------------------------------------------------------*/
double carrev3(double* a)
{
    return(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]);
}

/*---------------------------------------------------------------------
	Longueur d'un vecteurs de R3 : s = sqrt(A'*A)
	---------------------------------------------

	Entrees:
		double *A	: Adresse vecteur donné

	Sorties:
		Retour		: Longueur du vecteur
----------------------------------------------------------------------*/
double longv3(double* a)
{
    return(sqrt(a[0]*a[0]+a[1]*a[1]+a[2]*a[2]));
}

/*---------------------------------------------------------------------
	Normalisation d'un vecteur de R3 : A = B / ||B||
	------------------------------------------------

	Entrees:
		double *A	: Adresse vecteur unitaire résultat
		double *B	: Adresse vecteur donné

	Sorties:
		A est affecté
		Retour		: Adresse vecteur unitaire résultat

	Remarque : A et B peuvent occuper la même mémoire
		       si ||B|| = 0  -> A[0] = 1, les autres nuls.

	Remarque : A peut occuper la même mémoire que B
----------------------------------------------------------------------*/
double *unitv3(double* a, double* b)
{
    double l ;
    l = sqrt(b[0]*b[0]+b[1]*b[1]+b[2]*b[2]);
    if (l == 0.)
    {
        a[0]=1.; a[1]=0.; a[2]=0.;
    }
    else
    {
        a[0] = b[0]/l;
		a[1] = b[1]/l;
        a[2] = b[2]/l;
    }
    return a;
}

/*---------------------------------------------------------------------
	Normalisation d'un vecteur de R3 : A = B / ||B||
	------------------------------------------------

	Entrees:
		double *A	: Adresse vecteur unitaire résultat
		double *B	: Adresse vecteur donné

	Sorties:
		A est affecté
		Retour		: Longueur du vecteur B

	Remarque : A et B peuvent occuper la même mémoire
		       si ||B|| = 0  -> A[0] = 1, les autres nuls.

	Remarque : A peut occuper la même mémoire que B
----------------------------------------------------------------------*/
double lunitv3(double* a, double* b)
{
    double l ;
    l = sqrt(b[0]*b[0]+b[1]*b[1]+b[2]*b[2]);
    if (l == 0.)
    {
        a[0]=1.; a[1]=0.; a[2]=0.;
    }
    else
    {
        a[0] = b[0]/l;
		a[1] = b[1]/l;
        a[2] = b[2]/l;
    }
    return(l);
}

/*---------------------------------------------------------------------
	Affection d'un scalaire aux composantes d'une matrice de R3
	-----------------------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice m3 affectée
		double s    : Scalaire à affecter

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice m3 affectée

----------------------------------------------------------------------*/
double *setm3(double* A, double s)
{
int i ; for(i=0;i<9;i++) A[i]=s;
	return A;
}

/*---------------------------------------------------------------------
	Recopie d'une matrice de R3 : A = B
	-----------------------------------

	Entrees:
		double *A	: Adresse de la matrice m3 affectée
		double *B	: Adresse de la matrice m3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice m3 affectée
----------------------------------------------------------------------*/
double *cpym3(double* A, double* B)
{
int i ; for(i=0;i<9;i++) A[i]=B[i];
	return A;
}

/*---------------------------------------------------------------------
	Négation d'une matrice de R3 : A = - B
	--------------------------------------

	Entrees:
		double *A	: Adresse de la matrice m3 affectée
		double *B	: Adresse de la matrice m3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice m3 affectée

    Remarque : A et B peuvent occuper la même mémoire.
----------------------------------------------------------------------*/
double *negm3(double* A, double* B)
{
int i ; for(i=0;i<9;i++) A[i]=-B[i];
	return A;
}

/*---------------------------------------------------------------------
    Multiplication d'une matrice de R3 par un scalaire : A = sB
	-----------------------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice m3 affectée
		double *B	: Adresse de la matrice m3 donnée
		double s	: Scalaire donné

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice m3 affectée

    Remarque : A et B peuvent occuper la même mémoire.
----------------------------------------------------------------------*/
double *scam3(double* A, double s, double* B)
{
int i ; for(i=0;i<9;i++) A[i]=s*B[i];
	return A;
}

/*---------------------------------------------------------------------
	Addition de matrices de R3 : A = B + C
	--------------------------------------

	Entrees:
		double *A	: Adresse de la matrice m3 affectée
		double *B	: Adresse de la première matrice m3 donnée
		double *C	: Adresse de la deuxième matrice m3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice m3 affectée

    Remarque : A peut occuper la même mémoire que B ou C
----------------------------------------------------------------------*/
double *addm3(double* A, double* B, double *C)
{
int i ; for(i=0;i<9;i++) A[i]=B[i]+C[i];
	return A;
}

/*---------------------------------------------------------------------
Soustraction de matrices de R3 : A = B - C
------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice m3 affectée
		double *B	: Adresse de la première matrice m3 donnée
		double *C	: Adresse de la deuxième matrice m3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice m3 affectée

	Remarque : A peut occuper la même mémoire que B ou C
----------------------------------------------------------------------*/
double *subm3(double* A, double* B, double *C)
{
int i ; for(i=0;i<9;i++) A[i]=B[i]-C[i];
	return A;
}
/*---------------------------------------------------------------------
	Cumulation de matrices de R3 : A = B + sC
	-----------------------------------------

	Entrees:
		double *A	: Adresse de la matrice m3 affectée
		double *B	: Adresse de la première matrice m3 donnée
		double s	: Scalaire donné
		double *C	: Adresse de la deuxième matrice m3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice m3 affectée

	Remarque : A peut occuper la même mémoire que B ou C
----------------------------------------------------------------------*/
double *cumm3(double* A, double* B, double s, double *C)
{
int i ; for(i=0;i<9;i++) A[i]=B[i]+s*C[i];
	return A;
}

/*---------------------------------------------------------------------
	Matrice Identite de R3 :  A = I
	-------------------------------

	Entrees:
		double *A	: Adresse de la matrice affectée

    Sorties:
		A est affectée
		Retour		: Adresse de A
----------------------------------------------------------------------*/
double *eyem3(double *a)
{
    a[0] = a[4] = a[8] = 1. ;
    a[1] = a[2] = a[3] = 0. ;
    a[5] = a[6] = a[7] = 0. ;
	return a ;
}

/*---------------------------------------------------------------------
	Génération d'une matrice diagonale à partir d'un vecteur de R3
	--------------------------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice affectée
		double *V	: Adresse du vecteur donné

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice affectée
----------------------------------------------------------------------*/
double *diag2m3(double *a, double *v)
{
    a[0] = v[0]; a[3] = 0.  ; a[6] = 0.  ;
    a[1] = 0.  ; a[4] = v[1]; a[7] = 0.  ;
    a[2] = 0.  ; a[5] = 0.  ; a[8] = v[2];
	return a ;
}

/*---------------------------------------------------------------------
	Transposée d'une matrice de R3 : A = A'
	--------------------------------------

	Entrees:
		double *A	: Adresse de la matrice

	Sorties:
		A est transposée
		Retour		: Adresse de A
----------------------------------------------------------------------*/
double *tm3(double *a)
{
double s ;
	s = a[3]; a[3] = a[1]; a[1] = s;
	s = a[6]; a[6] = a[2]; a[2] = s;
	s = a[7]; a[7] = a[5]; a[5] = s;
	return a ;
}

/*---------------------------------------------------------------------
	Extraction du vecteur diagonal d'une matrice de R3
	--------------------------------------------------

	Entrees:
		double *V	: Adresse vecteur affecté
		double *A	: Adresse de la matrice donnée

	Sorties:
		V est affecté
		Retour		: Adresse vecteur affecté
----------------------------------------------------------------------*/
double *m3todiag(double *v, double *a)
{
    v[0] = a[0];
    v[1] = a[4];
	v[2] = a[8];
	return v ;
}

/*---------------------------------------------------------------------
	Génération matrice antisymétrique à partir d'un vecteur de R3
	-------------------------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice affectée
		double *V	: Adresse vecteur donné

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice affectée
----------------------------------------------------------------------*/
double *v3toasm3(double *a, double *v)
{
    a[0] =   0. ; a[3] = -v[2]; a[6] =  v[1];
    a[1] =  v[2]; a[4] =  0.  ; a[7] = -v[0];
    a[2] = -v[1]; a[5] =  v[0]; a[8] =   0. ;
	return a ;
}

/*---------------------------------------------------------------------
	Extraction du vecteur antisymétrique d'une matrice de R3
	--------------------------------------------------------

	Entrees:
		double *V	: Adresse vecteur affecté
		double *A	: Adresse de la matrice donné

	Sorties:
		V est affecté
		Retour		: Adresse vecteur affecté
----------------------------------------------------------------------*/
double *asm3tov3(double *v, double *a)
{
    v[0] = 0.5*(a[5]-a[7]);
    v[1] = 0.5*(a[6]-a[2]);
	v[2] = 0.5*(a[1]-a[3]);
	return v ;
}

/*---------------------------------------------------------------------
	Produit matrice x vecteurs de R3 : w = A v
	-----------------------------------------------

	Entrees:
		double *w	: Adresse du vecteur affecté
		double *A	: Adresse de la matrice de R3 donné
		double *v	: Adresse du vecteur de R3 donné

	Sorties:
		w est affecté
		Retour		: Adresse du vecteur affecté

    Remarque : w et v peuvent occuper la même mémoire.
----------------------------------------------------------------------*/
double *m3v3(double* w, double* a, double* v)
{
double p, q ;
      p  = a[0] * v[0] + a[3] * v[1] + a[6] * v[2];
      q  = a[1] * v[0] + a[4] * v[1] + a[7] * v[2];
    w[2] = a[2] * v[0] + a[5] * v[1] + a[8] * v[2];
    w[0] = p ;
    w[1] = q ;
	return w;
}

/*---------------------------------------------------------------------
	Produit matrice transposée x vecteurs de R3 : w = A' v
	------------------------------------------------------

	Entrees:
		double *w	: Adresse du vecteur affecté
		double *A	: Adresse de la matrice de R3 donné
		double *v	: Adresse du vecteur de R3 donné

	Sorties:
		w est affecté
		Retour		: Adresse du vecteur affecté

    Remarque : w et v peuvent occuper la même mémoire.
----------------------------------------------------------------------*/
double *tm3v3(double* w, double* a, double* v)
{
double p, q ;
      p  = a[0] * v[0] + a[1] * v[1] + a[2] * v[2];
      q  = a[3] * v[0] + a[4] * v[1] + a[5] * v[2];
    w[2] = a[6] * v[0] + a[7] * v[1] + a[8] * v[2];
    w[0] = p ;
    w[1] = q ;
	return w;
}

/*---------------------------------------------------------------------
	Produit vecteur transposé x matrice de R3 : a' = b' C
	------------------------------------------------------

	Entrees:
		double *a	: Adresse du vecteur affecté
		double *b	: Adresse du vecteur de R3 donné
		double *c	: Adresse de la matrice de R3 donné

	Sorties:
		a est affecté
		Retour		: Adresse du vecteur affecté

    Remarque : a et b peuvent occuper la même mémoire.
----------------------------------------------------------------------*/
double *tv3m3( double* a,double* b, double* c)
{
double d,e ;
    d    = b[0] * c[0] + b[1] * c[1] + b[2] * c[2];
    e    = b[0] * c[3] + b[1] * c[4] + b[2] * c[5];
    a[2] = b[0] * c[6] + b[1] * c[7] + b[2] * c[8];
	a[1] = e; a[0] = d;
	return a;
}

/*---------------------------------------------------------------------
	Produit matrice x matrice de R3 : A = B C
	-----------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice affectée
		double *B	: Adresse de la première matrice de R3 donnée
		double *C	: Adresse de la deuxième matrice de R3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice affectée

    Remarque : A peut occuper la même mémoire que B ou C
----------------------------------------------------------------------*/
double *m3m3(double* a,double* b, double* c)
{
register int i ;
double d[8] ;
    d[0] = b[0] * c[0] + b[3] * c[1] + b[6] * c[2];
    d[1] = b[1] * c[0] + b[4] * c[1] + b[7] * c[2];
    d[2] = b[2] * c[0] + b[5] * c[1] + b[8] * c[2];

    d[3] = b[0] * c[3] + b[3] * c[4] + b[6] * c[5];
    d[4] = b[1] * c[3] + b[4] * c[4] + b[7] * c[5];
    d[5] = b[2] * c[3] + b[5] * c[4] + b[8] * c[5];

    d[6] = b[0] * c[6] + b[3] * c[7] + b[6] * c[8];
    d[7] = b[1] * c[6] + b[4] * c[7] + b[7] * c[8];
    a[8] = b[2] * c[6] + b[5] * c[7] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
	return a;
}

/*---------------------------------------------------------------------
	Produit matrice x matrice transposée de R3 : A = B C'
	----------------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice affectée
		double *B	: Adresse de la première matrice de R3 donnée
		double *C	: Adresse de la deuxième matrice de R3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice affectée

    Remarque : A peut occuper la même mémoire que B ou C
----------------------------------------------------------------------*/
double *m3tm3(double* a, double* b, double* c)
{
register int i ;
double d[8] ;
    d[0] = b[0] * c[0] + b[3] * c[3] + b[6] * c[6];
    d[1] = b[1] * c[0] + b[4] * c[3] + b[7] * c[6];
    d[2] = b[2] * c[0] + b[5] * c[3] + b[8] * c[6];

    d[3] = b[0] * c[1] + b[3] * c[4] + b[6] * c[7];
    d[4] = b[1] * c[1] + b[4] * c[4] + b[7] * c[7];
    d[5] = b[2] * c[1] + b[5] * c[4] + b[8] * c[7];

    d[6] = b[0] * c[2] + b[3] * c[5] + b[6] * c[8];
    d[7] = b[1] * c[2] + b[4] * c[5] + b[7] * c[8];
    a[8] = b[2] * c[2] + b[5] * c[5] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
	return a;
}

/*---------------------------------------------------------------------
	Produit matrice transposée x matrice de R3 : A = B'C
	----------------------------------------------------

	Entrees:
		double *A	: Adresse de la matrice affectée
		double *B	: Adresse de la première matrice de R3 donnée
		double *C	: Adresse de la deuxième matrice de R3 donnée

	Sorties:
		A est affectée
		Retour		: Adresse de la matrice affectée

    Remarque : A peut occuper la même mémoire que B ou C
----------------------------------------------------------------------*/
double *tm3m3( double* a,double* b, double* c)
{
register int i;
double d[8] ;
    d[0] = b[0] * c[0] + b[1] * c[1] + b[2] * c[2];
    d[1] = b[3] * c[0] + b[4] * c[1] + b[5] * c[2];
    d[2] = b[6] * c[0] + b[7] * c[1] + b[8] * c[2];

    d[3] = b[0] * c[3] + b[1] * c[4] + b[2] * c[5];
    d[4] = b[3] * c[3] + b[4] * c[4] + b[5] * c[5];
    d[5] = b[6] * c[3] + b[7] * c[4] + b[8] * c[5];

    d[6] = b[0] * c[6] + b[1] * c[7] + b[2] * c[8];
    d[7] = b[3] * c[6] + b[4] * c[7] + b[5] * c[8];
    a[8] = b[6] * c[6] + b[7] * c[7] + b[8] * c[8];
    for(i = 0 ; i < 8 ; ++i) a[i] = d[i] ;
	return a;
}

/*---------------------------------------------------------------------
	Trace d'une matrice de R3 : s = Sigma de Aii
	--------------------------------------------

	Entrees:
		double *A	: Adresse matrice donnée

	Sorties:
		Retour		: trace de la matrice
----------------------------------------------------------------------*/
double tracem3(double* a)
{
    return(a[0]+a[4]+a[8]);
}

/*---------------------------------------------------------------------
	Déterminant d'une matrice de R3 : s = |A|
	-----------------------------------------

	Entrees:
		double *A	: Adresse matrice donnée

	Sorties:
		Retour		: déterminant de la matrice
----------------------------------------------------------------------*/
double detm3(double* a)
{
    return(a[0]*(a[4]*a[8]-a[7]*a[5])
		  +a[3]*(a[7]*a[2]-a[1]*a[8])
		  +a[6]*(a[1]*a[5]-a[4]*a[2]));
}

/*---------------------------------------------------------------------
	Inverse d'une matrice de R3 : inv(A)
	------------------------------------

	Entrees:
		double *A	: Adresse matrice

	Sorties:
	    A = inv(A) si det != 0.
	Retour			: Adresse de A

   ATTENTION : EXTREMEMENT DANGEREUX SI DETERMINANT NUL
   FAIRE PRECEDER DU CALCUL DU DETERMINANT
----------------------------------------------------------------------*/
double *invm3(double* a)
{
double delta ;
double c[9] ;

    crossv3 (c, a+3, a+6) ;
    delta = dotv3(a, c) ;
    if (delta == 0.) return a ;
    crossv3 (c+3, a+6, a) ;
    crossv3 (c+6, a, a+3) ;
    tm3(scam3 (a, 1./delta, c)) ;
    return a ;
}

/*---------------------------------------------------------------------
	Déterminant et Inverse d'une matrice de R3 : s = |A|, inv(A)
	------------------------------------------------------------

	Entrees:
		double *A	: Adresse matrice

	Sorties:
	    A = inv(A) si det != 0.
		Retour		: déterminant de la matrice
----------------------------------------------------------------------*/
double dinvm3(double* a)
{
double delta ;
double c[9] ;

    crossv3 (c, a+3, a+6) ;
    delta = dotv3(a, c) ;
    if (delta == 0.) return delta ;
    crossv3 (c+3, a+6, a) ;
    crossv3 (c+6, a, a+3) ;
    tm3(scam3 (a, 1./delta, c)) ;
    return delta ;
}

/*---------------------------------------------------------------------
	Impression d'une matrice de R3
	------------------------------

	Entrees:
		char *s		: Texte à impimer (RC ajouté en fin du texte)
		double *A	: Adresse matrice donnée à imprimer
----------------------------------------------------------------------*/
void printm3(char *s, double *a)
{
int i, j ;

	sprintf(strfmt+2,"%d.%dl%c",(printformat/10)%100,(printformat%10),
		printformat/1000 ? 'g' : 'f');
	printf("%s\n",s);
	for (i = 0 ; i < 3 ; i++)
	{
		for(j=i ; j<9 ; j += 3) printf(strfmt,a[j]) ; printf("\n");
	}
}

/*---------------------------------------------------------------------
        Matrice d'attitude d'un repère de visée
   ------------------------------------------------------------

Entrees:
    double azi	: Azimut de la ligne de visée
    double sit  : Site de la ligne de visée

Sorties:
    mat : la matrice de passage du repère fixe au repère de visée

    Direction de visée selon -Z (l'axe +Z est dirigé vers l'oeil).
    l'axe X est horizontal et Y est dans le plan vertical contenant Z
    Y dirigé vers le haut, si |sit] < PI/2
----------------------------------------------------------------------*/
void asiTom3(double *mat, double azi, double sit)
{
    double ca, sa, cs, ss;
    ca = cos(azi); sa = sin(azi);
    cs = cos(sit); ss = sin(sit);

    mat[0] = sa;    mat[3] =-ca*ss; mat[6] =-ca*cs;
    mat[1] = -ca;   mat[4] =-sa*ss; mat[7] =-sa*cs;
    mat[2] = 0.;    mat[5] = cs;    mat[8] =-ss;
}


/*---------------------------------------------------------------------
        Matrice d'attitue d'un repère local sur Terre
   ------------------------------------------------------------

Entrees:
    double lon	: longitude / greenwich
    double lat  : latitude géodésique

Sorties:
    mat : la matrice de passage du repère de Greenwich au repère local
    [GA] telle que V_g = [GA] V_a

    Pour le repère local Ra, Z est vers le zénit, X vers le Nord et
    Y est vers l'ouest
----------------------------------------------------------------------*/
void lolaTom3(double *mat, double lon, double lat)
{
    double cl = cos(lon);
    double sl = sin(lon);
    double sf = sin(lat);
    double cf = cos(lat);
    mat[0] = -cl*sf; mat[3] =  sl; mat[6] = cl*cf;
    mat[1] = -sl*sf; mat[4] = -cl; mat[7 ]= sl*cf;
    mat[2] =     cf; mat[5] =  0.; mat[8] =    sf;
}

/*---------------------------------------------------------------------
        Matrice d'attitue d'un repère local sur Terre
   ------------------------------------------------------------

Entrees:
    double lon	: longitude / greenwich
    double lat  : latitude géodésique

Sorties:
    mat : la matrice de passage du repère de Greenwich au repère local
    [GA] telle que V_g = [GA] V_a

    Pour le repère local Ra, Z est vers le zénit, X vers le Sud et
    Y est vers l'est
----------------------------------------------------------------------*/
void lolaAstroTom3(double *mat, double lon, double lat)
{
    double cl = cos(lon);
    double sl = sin(lon);
    double sf = sin(lat);
    double cf = cos(lat);
    mat[0] =  cl*sf; mat[3] = -sl; mat[6] = cl*cf;
    mat[1] =  sl*sf; mat[4] =  cl; mat[7 ]= sl*cf;
    mat[2] =    -cf; mat[5] =  0.; mat[8] =    sf;
}

/*---------------------------------------------------------------------------------------------------
  Conversion matrice 3x3 MLL en matrice homogène 4x4 type openGL
  ---------------------------------------------------------------------------------------------------
  Entree:
    double *mat   : matrice 3x3 dans mat[9] rangement MLL

  Sortie:
    double *m44   : matrice homogene 4x4 dans m44[16] rangement openGL (en colonne, idem MLL)
 ---------------------------------------------------------------------------------------------------*/
double *m3ToGLmat44(double *m44,double *mat)
{
    m44[0] = mat[0] ; m44[4] = mat[3] ;  m44[8]  = mat[ 6] ; m44[12] = 0. ;
    m44[1] = mat[1] ; m44[5] = mat[4] ;  m44[9]  = mat[ 7] ; m44[13] = 0. ;
    m44[2] = mat[2] ; m44[6] = mat[5] ;  m44[10] = mat[ 8] ; m44[14] = 0. ;
    m44[3] = 0.     ; m44[7] = 0.     ;  m44[11] = 0.      ; m44[15] = 1. ;
    return m44 ;
}

/*---------------------------------------------------------------------------------------------------
  Conversion matrice 3x3 MLL en matrice homogène 4x4 type openGL, avec transposition
  ---------------------------------------------------------------------------------------------------
  Entree:
    double *mat   : matrice 3x3 dans mat[9] rangement MLL

  Sortie:
    double *m44   : matrice homogene 4x4 dans m44[16] rangement openGL (en colonne, idem MLL)
  La transposée de la 3x3 est copiée dans la matrice openGL
 ---------------------------------------------------------------------------------------------------*/
double *tm3ToGLmat44(double *m44,double *mat)
{
    m44[0] = mat[0] ; m44[4] = mat[1] ;  m44[8]  = mat[ 2] ; m44[12] = 0. ;
    m44[1] = mat[3] ; m44[5] = mat[4] ;  m44[9]  = mat[ 5] ; m44[13] = 0. ;
    m44[2] = mat[6] ; m44[6] = mat[7] ;  m44[10] = mat[ 8] ; m44[14] = 0. ;
    m44[3] = 0.     ; m44[7] = 0.     ;  m44[11] = 0.      ; m44[15] = 1. ;
    return m44 ;
}
